"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TestManagedApi = void 0;
const __1 = require("..");
const CollectionUtil_1 = require("../Util/CollectionUtil");
const p2r = require("path-to-regexp");
const StreamCoercer_1 = require("../Util/StreamCoercer");
class TestManagedApi extends __1.ManagedApi {
    constructor() {
        super(...arguments);
        this.handlers = new Map();
    }
    init() {
        const handlers = this.initHandlers();
        for (const [method, routes] of handlers) {
            for (const [route, instance] of routes) {
                CollectionUtil_1.CollectionUtil.addToMapSet(this.handlers, method, {
                    route,
                    matchFunc: p2r.match(route),
                    ...instance,
                });
            }
        }
        return this;
    }
    async invokeApiCall(method, route, params) {
        const result = this.getHandlerInstance(method, route);
        if (!result) {
            return this.getResponse(404);
        }
        return result.instance.wrappedHandler({
            ...params,
            pathParams: result.matchResult.params,
            bodyContents: (params.bodyContents
                ?
                    {
                        contentsStream: this.underfinedOrNull(params.bodyContents.contentsStream),
                        readStreamToStringAsync: this.underfinedOrNull(params.bodyContents.readStreamToStringAsync),
                        readStreamToStringCb: this.underfinedOrNull(params.bodyContents.readStreamToStringCb),
                        streamContentsMimeRaw: this.underfinedOrNull(params.bodyContents.streamContentsMimeRaw),
                        streamContentsMimeType: this.underfinedOrNull(params.bodyContents.streamContentsMimeType),
                        parsedBody: params.bodyContents.parsedBody,
                    }
                : undefined),
        });
    }
    underfinedOrNull(val) {
        return typeof val === 'undefined' ? null : val;
    }
    getHandlerInstance(method, route) {
        if (!this.handlers.has(method)) {
            return;
        }
        for (const instance of this.handlers.get(method)) {
            let matchResult = instance.matchFunc(route);
            if (!matchResult) {
                continue;
            }
            return { matchResult, instance };
        }
    }
    getResponse(code, description = '') {
        return {
            streamMode: StreamCoercer_1.StreamCoercionMode.None,
            statusCode: code,
            body: description,
            headers: {},
        };
    }
    setHeader(name, value) {
        throw new Error('NotImplemented');
    }
}
exports.TestManagedApi = TestManagedApi;
//# sourceMappingURL=TestTransport.js.map