import { ITransformerArguments } from '../transformer';
import { TransformerType } from '../Util/CompilationUtil';
export * from '../Util/CompilationUtil';
export * from '../Util/AsyncGlob';
export declare function getTransformer(): TransformerType;
export declare function getCompiledProgram(moduleNamesTs: string[], transformerArgs?: ITransformerArguments): any[];
export type ArgMatcherFunction<T> = (actual: T | undefined, ctx: IIncludeMatchContext) => boolean;
export declare function validateOrSetMatch<MatchType>(sym: Symbol, match: MatchType, ctx: IIncludeMatchContext): MatchType;
export interface IIncludeMatchContext {
    path?: string;
    funcmode?: 'matcher' | 'value';
    undefinedValMode?: 'explicit' | 'allowMissingKeys';
    symbols?: {
        [sym: symbol]: {
            matcher?: ArgMatcherFunction<any>;
            value?: any;
        };
    };
}
/**
 * Asserts that an object deep includes the properties
 * @param actual The actual object observed
 * @param expected The expected object
 * @param path The current path (omit)
 * @param funcmode The mode for evaluating expected values that are functions. When `value` the function is treated as the value that is expected. When `matcher` the function is considered a test function and executed, with the first argument being the actual value. It should return true if the value matches expectation
 */
export declare function assertRealInclude(actual: object | undefined, expected: object, path?: string, ctx?: IIncludeMatchContext): void;
/**
 * Asserts that an array deep includes the children
 * @param actual The actual array observed
 * @param expected The expected array
 * @param path The current path (omit)
 * @param funcmode The mode for evaluating expected values that are functions. When `value` the function is treated as the value that is expected. When `matcher` the function is considered a test function and executed, with the first argument being the actual value. It should return true if the value matches expectation
 */
export declare function assertRealArrayInclude(actual: Array<any>, expected: Array<any>, path?: string, ctx?: IIncludeMatchContext): void;
