"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertRealArrayInclude = exports.assertRealInclude = exports.validateOrSetMatch = exports.getCompiledProgram = exports.getTransformer = void 0;
const ts = require("typescript");
const transformer_1 = require("../transformer");
const CompilationUtil_1 = require("../Util/CompilationUtil");
const chai_1 = require("chai");
__exportStar(require("../Util/CompilationUtil"), exports);
__exportStar(require("../Util/AsyncGlob"), exports);
function getTransformer() {
    return transformer_1.default;
}
exports.getTransformer = getTransformer;
function getCompiledProgram(moduleNamesTs, transformerArgs) {
    const jsModuleNames = moduleNamesTs.map(ts => ts.replace(/\.ts$/, '.js'));
    const options = (0, CompilationUtil_1.getDefaultCompilerOptions)();
    const program = ts.createProgram(moduleNamesTs, options);
    const before = [getTransformer()(program, transformerArgs)];
    for (let i = 0; i < moduleNamesTs.length; ++i) {
        program.emit(program.getSourceFile(moduleNamesTs[i]), undefined, undefined, false, {
            before,
        });
    }
    return jsModuleNames.map(js => require(js).default);
}
exports.getCompiledProgram = getCompiledProgram;
function validateOrSetMatch(sym, match, ctx) {
    const symval = ctx.symbols[sym];
    if (!symval.matcher(symval.value, ctx)) {
        // Symbol hasn't been assigned a value yet. Try and find a matching key
        if (symval.matcher(match, ctx)) {
            symval.value = match;
        }
        else {
            chai_1.assert.fail(`Property did not pass symbol ${sym.toString()} matcher ${match}`);
        }
    }
    else {
        // Symbol has resolved a value, check it
        chai_1.assert.equal(match, symval.value, `Property symbol ${sym.toString()} did not match expected value ${symval.value}`);
    }
    return match;
}
exports.validateOrSetMatch = validateOrSetMatch;
function initContextDefaults(ctx) {
    const newCtx = {
        funcmode: 'value',
        undefinedValMode: 'allowMissingKeys',
        ...ctx,
        symbols: {},
    };
    if (ctx.symbols) {
        for (const sym of Reflect.ownKeys(ctx.symbols)) {
            newCtx.symbols[sym] = {
                matcher: (a) => typeof a !== 'undefined',
                value: undefined,
                ...ctx.symbols[sym],
            };
        }
    }
    return newCtx;
}
/**
 * Asserts that an object deep includes the properties
 * @param actual The actual object observed
 * @param expected The expected object
 * @param path The current path (omit)
 * @param funcmode The mode for evaluating expected values that are functions. When `value` the function is treated as the value that is expected. When `matcher` the function is considered a test function and executed, with the first argument being the actual value. It should return true if the value matches expectation
 */
function assertRealInclude(actual, expected, path = '$', ctx = {}) {
    ctx = initContextDefaults(ctx);
    if (typeof actual !== typeof expected) {
        chai_1.assert.fail(`Expected type ${typeof expected} at ${path} but got ${typeof actual}`);
    }
    const matchedKeys = new Set();
    for (const expectedKey of [...Object.keys(expected), ...Object.getOwnPropertySymbols(expected)]) {
        let actualKey = expectedKey;
        const propPath = `${path}.${expectedKey.toString()}`;
        const expectedVal = expected[expectedKey];
        if (typeof expectedKey === 'symbol' && expectedKey in ctx.symbols) {
            const symval = ctx.symbols[expectedKey];
            if (!symval.matcher(symval.value, ctx)) {
                // Symbol hasn't been assigned a value yet. Try and find a matching key
                const foundkey = Object.keys(actual).find(v => !matchedKeys.has(v) && symval.matcher(v, ctx));
                if (symval.matcher(foundkey, ctx)) {
                    symval.value = foundkey;
                }
                else {
                    chai_1.assert.fail(`Could not resolve property ${propPath}`);
                }
            }
            else {
                // Symbol has resolved a value, try and find it
                chai_1.assert.property(actual, symval.value, `Expected resolved symbol ${propPath} as key (resolved to value ${symval.value})`);
            }
            actualKey = symval.value;
        }
        else {
            if (ctx.undefinedValMode === 'allowMissingKeys' && typeof expectedVal === 'undefined') {
                chai_1.assert.isUndefined(actual[expectedKey], `Expected property ${propPath} to not exist or be undefined`);
            }
            else {
                chai_1.assert.property(actual, expectedKey, `Expected property ${propPath}`);
            }
        }
        if (typeof expectedVal === 'object') {
            if (Array.isArray(expectedVal)) {
                assertRealArrayInclude(actual[actualKey], expectedVal, propPath, ctx);
            }
            else {
                assertRealInclude(actual[actualKey], expectedVal, propPath, ctx);
            }
        }
        else if (typeof expectedVal === 'function' && ctx.funcmode === 'matcher') {
            chai_1.assert.isTrue(expectedVal(actual[actualKey], ctx), `Matcher function did not match expected value "${expectedVal}" at ${path}`);
        }
        else if (typeof expectedVal === 'symbol' && expectedVal in ctx.symbols) {
            const symval = ctx.symbols[expectedKey];
            if (!symval.matcher(symval.value, ctx)) {
                // Symbol hasn't been assigned a value yet. Try and find a matching key
                if (symval.matcher(actual[actualKey], ctx)) {
                    symval.value = actual[actualKey];
                }
                else {
                    chai_1.assert.fail(`Property did not pass symbol ${expectedVal.toString()} matcher ${propPath}`);
                }
            }
            else {
                // Symbol has resolved a value, check it
                chai_1.assert.equal(actual[actualKey], symval.value, `Property symbol ${propPath} did not match expected value ${symval.value}`);
            }
        }
        else {
            chai_1.assert.equal(actual[actualKey], expectedVal, `Property does not match expected value: ${path}`);
        }
        matchedKeys.add(actualKey);
    }
}
exports.assertRealInclude = assertRealInclude;
/**
 * Asserts that an array deep includes the children
 * @param actual The actual array observed
 * @param expected The expected array
 * @param path The current path (omit)
 * @param funcmode The mode for evaluating expected values that are functions. When `value` the function is treated as the value that is expected. When `matcher` the function is considered a test function and executed, with the first argument being the actual value. It should return true if the value matches expectation
 */
function assertRealArrayInclude(actual, expected, path = '$', ctx = {}) {
    ctx = initContextDefaults(ctx);
    const skipActuals = new Set();
    for (const expectedIndex in expected) {
        const expectedVal = expected[expectedIndex];
        const expectedType = typeof expectedVal;
        const expectedPath = `${path}.${expectedIndex}`;
        let found = false;
        for (const actualIndex in actual) {
            if (skipActuals.has(actualIndex)) {
                continue;
            }
            if (expectedType === 'function' && ctx.funcmode === 'matcher') {
                if (expectedVal(actual[actualIndex], ctx)) {
                    skipActuals.add(actualIndex);
                    found = true;
                    break;
                }
            }
            if (expectedType === 'symbol' && expectedVal in ctx.symbols) {
                const symval = ctx.symbols[expectedVal];
                if (!symval.matcher(symval.value, ctx)) {
                    // Symbol hasn't been assigned a value yet. Try and find a matching key
                    if (symval.matcher(actual[actualIndex], ctx)) {
                        symval.value = actual[actualIndex];
                    }
                    else {
                        chai_1.assert.fail(`Property did not pass symbol ${expectedVal.toString()} matcher ${expectedPath}`);
                    }
                }
                else {
                    // Symbol has resolved a value, check it
                    chai_1.assert.equal(actual[actualIndex], symval.value, `Property symbol ${expectedPath} did not match expected value ${symval.value}`);
                }
                skipActuals.add(actualIndex);
                found = true;
                break;
            }
            if (typeof actual[actualIndex] !== expectedType) {
                continue;
            }
            if (expectedType === 'object') {
                try {
                    if (Array.isArray(expectedVal)) {
                        assertRealArrayInclude(actual[actualIndex], expectedVal, expectedPath, ctx);
                    }
                    else {
                        assertRealInclude(actual[actualIndex], expectedVal, expectedPath, ctx);
                    }
                    skipActuals.add(actualIndex);
                    found = true;
                    break;
                }
                catch (e) {
                    // Do nothing
                }
            }
            else if (expectedVal === actual[actualIndex]) {
                skipActuals.add(actualIndex);
                found = true;
                break;
            }
        }
        if (!found) {
            chai_1.assert.fail(undefined, expectedVal, `Could not find array member value at path: ${expectedPath}`);
        }
    }
}
exports.assertRealArrayInclude = assertRealArrayInclude;
//# sourceMappingURL=TestUtil.js.map