import { ApiMethod } from "..";
import { IHandlerTreeNodeHandler, IHandlerTreeNodeParameter, IHandlerTreeNodeRoot, IHandlerTreeNodeHandlerCollection } from "../transformer/HandlerTree";
import { IMetadataDescriptor, ITransformerMetadata } from "../transformer/TransformerMetadata";
import { IApiParamDefinition } from "../apiManagement/ApiDefinition";
import { ArgMatcherFunction, IIncludeMatchContext } from "./TestUtil";
type OptionalPropertyMatcher<T> = T | ArgMatcherFunction<T>;
export type DeepPartial<T> = OptionalPropertyMatcher<{
    [P in keyof T]?: T[P] extends Array<infer U> ? OptionalPropertyMatcher<Array<DeepPartial<U>>> : T[P] extends ReadonlyArray<infer U> ? OptionalPropertyMatcher<ReadonlyArray<DeepPartial<U>>> : OptionalPropertyMatcher<DeepPartial<T[P]>>;
}>;
/**
 * During type testing, we encounter paths like '#/definitions/{definitionName}.<some random hex characters>'
 *
 * We wanted to check that the whole string starts with `#/definitions/{definitionName}` and then memorize everything after `#/definitions/` and assign it to the `symbol` in the context
 * checker for the real include checker.
 * @param value
 * @param symbol
 * @param definitionName
 * @param ctx
 * @param defRoot
 * @returns
 */
export declare function definitionPathWithSymbolChecker(value: string, symbol: Symbol, definitionName: string, ctx: IIncludeMatchContext, defRoot?: string): boolean;
export declare function treeRootNode(children?: DeepPartial<IHandlerTreeNodeHandler['children']>, metadata?: DeepPartial<IHandlerTreeNodeHandler['metadata']>): DeepPartial<IHandlerTreeNodeRoot>;
export declare function treeHandlerMethodCollectionNode(children?: DeepPartial<IHandlerTreeNodeHandler['children']>, metadata?: DeepPartial<IHandlerTreeNodeHandler['metadata']>): DeepPartial<IHandlerTreeNodeHandlerCollection>;
export declare function treeHandlerMethodNode(method: ApiMethod, route: string, children?: DeepPartial<IHandlerTreeNodeHandler['children']>, metadata?: DeepPartial<IHandlerTreeNodeHandler['metadata']>): DeepPartial<IHandlerTreeNodeHandler>;
export declare function treeNodeMetadata(descriptor: DeepPartial<IMetadataDescriptor>, value?: any): DeepPartial<ITransformerMetadata>;
export declare function treeHandlerParameterNode(paramDef: DeepPartial<IApiParamDefinition>, children?: DeepPartial<IHandlerTreeNodeHandler['children']>, metadata?: DeepPartial<IHandlerTreeNodeHandler['metadata']>): DeepPartial<IHandlerTreeNodeParameter>;
export {};
