"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.treeHandlerParameterNode = exports.treeNodeMetadata = exports.treeHandlerMethodNode = exports.treeHandlerMethodCollectionNode = exports.treeRootNode = exports.definitionPathWithSymbolChecker = void 0;
const HandlerTree_1 = require("../transformer/HandlerTree");
const TestUtil_1 = require("./TestUtil");
/**
 * During type testing, we encounter paths like '#/definitions/{definitionName}.<some random hex characters>'
 *
 * We wanted to check that the whole string starts with `#/definitions/{definitionName}` and then memorize everything after `#/definitions/` and assign it to the `symbol` in the context
 * checker for the real include checker.
 * @param value
 * @param symbol
 * @param definitionName
 * @param ctx
 * @param defRoot
 * @returns
 */
function definitionPathWithSymbolChecker(value, symbol, definitionName, ctx, defRoot = '#/definitions/') {
    if (!value.startsWith(`${defRoot}${definitionName}`)) {
        return false;
    }
    (0, TestUtil_1.validateOrSetMatch)(symbol, value.substring(defRoot.length), ctx);
    return true;
}
exports.definitionPathWithSymbolChecker = definitionPathWithSymbolChecker;
function treeRootNode(children, metadata) {
    return {
        type: HandlerTree_1.HandlerTreeNodeType.Root,
        ...(children ? { children, } : {}),
        ...(metadata ? { metadata, } : {}),
    };
}
exports.treeRootNode = treeRootNode;
function treeHandlerMethodCollectionNode(children, metadata) {
    return {
        type: HandlerTree_1.HandlerTreeNodeType.HandlerCollection,
        ...(children ? { children, } : {}),
        ...(metadata ? { metadata, } : {}),
    };
}
exports.treeHandlerMethodCollectionNode = treeHandlerMethodCollectionNode;
function treeHandlerMethodNode(method, route, children, metadata) {
    return {
        apiMethod: method,
        route,
        type: HandlerTree_1.HandlerTreeNodeType.Handler,
        ...(children ? { children, } : {}),
        ...(metadata ? { metadata, } : {}),
    };
}
exports.treeHandlerMethodNode = treeHandlerMethodNode;
function treeNodeMetadata(descriptor, value) {
    return {
        ...descriptor,
        ...(value ? { value, } : {}),
    };
}
exports.treeNodeMetadata = treeNodeMetadata;
function treeHandlerParameterNode(paramDef, children, metadata) {
    return {
        paramDef: paramDef,
        type: HandlerTree_1.HandlerTreeNodeType.HandlerParameter,
        ...(children ? { children, } : {}),
        ...(metadata ? { metadata, } : {}),
    };
}
exports.treeHandlerParameterNode = treeHandlerParameterNode;
//# sourceMappingURL=TreeTestUtil.js.map