"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CollectionUtil = void 0;
class CollectionUtil {
    constructor() { }
    static addToMapSet(mapSet, addKey, addVal, kcompFunc, tcompFunc) {
        CollectionUtil.assertMapSet(mapSet);
        CollectionUtil.assertCompFunc(tcompFunc);
        CollectionUtil.assertCompFunc(kcompFunc);
        let set;
        if (kcompFunc) {
            for (const k of mapSet.keys()) {
                if (kcompFunc(k, addKey)) {
                    set = mapSet.get(k);
                    break;
                }
            }
            if (!set) {
                set = new Set();
                mapSet.set(addKey, set);
            }
        }
        else if (!mapSet.has(addKey)) {
            set = new Set();
            mapSet.set(addKey, set);
        }
        else {
            set = mapSet.get(addKey);
        }
        if (tcompFunc) {
            for (const entry of set) {
                if (tcompFunc(entry, addVal)) {
                    throw new Error('Generator/Decorator pair already registered');
                }
            }
        }
        else {
            if (set.has(addVal)) {
                throw new Error('Generator/Decorator pair already registered');
            }
        }
        set.add(addVal);
    }
    static mapSetContains(mapSet, addKey, addVal, kcompFunc, tcompFunc) {
        CollectionUtil.assertMapSet(mapSet);
        CollectionUtil.assertCompFunc(tcompFunc);
        CollectionUtil.assertCompFunc(kcompFunc);
        let set;
        if (kcompFunc) {
            for (const k of mapSet.keys()) {
                if (kcompFunc(k, addKey)) {
                    set = mapSet.get(k);
                    break;
                }
            }
            if (!set) {
                return false;
            }
        }
        else if (!mapSet.has(addKey)) {
            return false;
        }
        else {
            set = mapSet.get(addKey);
        }
        if (tcompFunc) {
            for (const entry of set) {
                if (tcompFunc(entry, addVal)) {
                    return true;
                }
            }
        }
        else {
            if (set.has(addVal)) {
                return true;
            }
        }
        return false;
    }
    static *iterateSet(mapSet, addKey, kcompFunc) {
        CollectionUtil.assertMapSet(mapSet);
        CollectionUtil.assertCompFunc(kcompFunc);
        if (kcompFunc) {
            for (const k of mapSet.keys()) {
                if (kcompFunc(k, addKey)) {
                    yield* mapSet.get(k).values();
                }
            }
        }
        else if (mapSet.has(addKey)) {
            yield* mapSet.get(addKey);
        }
    }
    static removeFromMapSet(mapSet, removeKey, removeVal, kcompFunc, tcompFunc) {
        CollectionUtil.assertMapSet(mapSet);
        CollectionUtil.assertCompFunc(tcompFunc);
        CollectionUtil.assertCompFunc(kcompFunc);
        let set;
        if (kcompFunc) {
            for (const k of mapSet.keys()) {
                if (kcompFunc(k, removeKey)) {
                    set = mapSet.get(k);
                    break;
                }
            }
            if (!set) {
                return;
            }
        }
        else if (!mapSet.has(removeKey)) {
            return;
        }
        else {
            set = mapSet.get(removeKey);
        }
        if (tcompFunc) {
            for (const entry of set) {
                if (tcompFunc(entry, removeVal)) {
                    CollectionUtil.removeEntry(mapSet, removeKey, set, removeVal);
                    return;
                }
            }
        }
        else {
            if (set.has(removeVal)) {
                CollectionUtil.removeEntry(mapSet, removeKey, set, removeVal);
                return;
            }
        }
    }
    static removeEntry(mapSet, removeKey, set, removeVal) {
        set.delete(removeVal);
        if (!set.size) {
            mapSet.delete(removeKey);
        }
    }
    static assertMapSet(mapSet) {
        if (!mapSet || !(mapSet instanceof Map)) {
            throw new Error('Invalid mapSet');
        }
    }
    static assertCompFunc(tcompFunc) {
        if (typeof tcompFunc !== 'function' && typeof tcompFunc !== 'undefined') {
            throw new Error('Invalid compFunc');
        }
    }
}
exports.CollectionUtil = CollectionUtil;
//# sourceMappingURL=CollectionUtil.js.map