"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compileSourceFile = exports.compileSources = exports.compileSourcesDir = exports.compileSourcesFromTsConfigFile = exports.parseTsConfig = exports.TsConfigParserHost = exports.getDefaultCompilerOptions = void 0;
const AsyncGlob_1 = require("./AsyncGlob");
const glob = require("glob");
const ts = require("typescript");
const fs = require("fs");
const path = require("path");
const fs_1 = require("fs");
function getDefaultCompilerOptions() {
    return {
        module: ts.ModuleKind.CommonJS,
        noEmitOnError: true,
        noImplicitAny: false,
        experimentalDecorators: true,
        target: ts.ScriptTarget.ES2019,
        downlevelIteration: true,
        sourceMap: true,
    };
}
exports.getDefaultCompilerOptions = getDefaultCompilerOptions;
class TsConfigParserHost {
    constructor() {
        this.useCaseSensitiveFileNames = true;
    }
    readDirectory(rootDir, extensions, excludes, includes, depth) {
        // TODO: Depth support
        const globPattern = `${rootDir}/**/*@(${extensions.join('|')})`;
        const opts = {
            ignore: excludes.map(e => this.pathToGlobIgnore(e, rootDir)),
            nocase: !this.useCaseSensitiveFileNames,
            absolute: true,
        };
        return glob.sync(globPattern, opts);
    }
    pathToGlobIgnore(p, basePath) {
        if (/\*|\.|\/|\\/.test(p)) {
            return path.join(basePath, p);
        }
        return path.join(basePath, p, '**/*');
    }
    fileExists(path) {
        return fs.existsSync(path);
    }
    readFile(path) {
        return fs.readFileSync(path, { encoding: 'utf8' });
    }
    trace(s) {
    }
}
exports.TsConfigParserHost = TsConfigParserHost;
function parseTsConfig(basePath, tsconfigPath) {
    const sourcefile = ts.readJsonConfigFile(tsconfigPath, (path) => (0, fs_1.readFileSync)(path, { encoding: 'utf8' }));
    return ts.parseJsonSourceFileConfigFileContent(sourcefile, new TsConfigParserHost(), basePath);
}
exports.parseTsConfig = parseTsConfig;
async function compileSourcesFromTsConfigFile(basePath, tsconfigPath, transformers) {
    // console.log({basePath, tsconfigPath});
    const config = parseTsConfig(basePath, tsconfigPath);
    // console.log({config});
    return compileSources(config.fileNames, {
        ...config.options,
        noEmit: true,
    }, transformers);
}
exports.compileSourcesFromTsConfigFile = compileSourcesFromTsConfigFile;
function resolveFilePathArray(files, basePath) {
    return files.map(f => path.resolve(f, basePath));
}
async function compileSourcesDir(path, options, transformers) {
    const matches = await (0, AsyncGlob_1.asyncGlob)(path);
    if (!matches || matches.length === 0) {
        throw new Error('No sources found to compile with glob: ' + path);
    }
    else {
        return compileSources(matches, options, transformers);
    }
}
exports.compileSourcesDir = compileSourcesDir;
function compileSources(matches, options, transformers) {
    const program = ts.createProgram(matches, options);
    const results = {};
    for (const file of matches) {
        results[file] = compileSourceFile(file, options, transformers, program);
    }
    return results;
}
exports.compileSources = compileSources;
function compileSourceFile(path, options, transformers, program) {
    if (!program) {
        program = ts.createProgram([path], options);
    }
    return ts.transform(program.getSourceFile(path), transformers.map(t => t(program)));
}
exports.compileSourceFile = compileSourceFile;
//# sourceMappingURL=CompilationUtil.js.map