"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StreamCoercer = exports.StreamCoercionMode = void 0;
const stream = require("stream");
const EventEmitter = require("events");
const StreamIntermediary_1 = require("./StreamIntermediary");
var StreamCoercionMode;
(function (StreamCoercionMode) {
    StreamCoercionMode[StreamCoercionMode["None"] = 0] = "None";
    StreamCoercionMode[StreamCoercionMode["InheritsStream"] = 1] = "InheritsStream";
    StreamCoercionMode[StreamCoercionMode["DuckPipe"] = 2] = "DuckPipe";
    StreamCoercionMode[StreamCoercionMode["Any"] = 7] = "Any";
    // TODO: Support EventEmitter streams
    // EventEmitter = 0b100,
})(StreamCoercionMode || (exports.StreamCoercionMode = StreamCoercionMode = {}));
class StreamCoercer {
    static IsCoercible(value, allowableModes) {
        if (typeof value === "object" && value) {
            for (const test of StreamCoercer.StreamCoercionModes) {
                if (allowableModes & test.mode && test.test(value)) {
                    return test.mode;
                }
            }
        }
        return StreamCoercionMode.None;
    }
    static CoerceWith(value, mode) {
        for (const test of StreamCoercer.StreamCoercionModes) {
            if (mode & test.mode) {
                return test.coerce(value);
            }
        }
        return undefined;
    }
}
exports.StreamCoercer = StreamCoercer;
StreamCoercer.DefaultOpts = {
    pipeOptions: { end: true },
};
StreamCoercer.StreamCoercionModes = [
    {
        mode: StreamCoercionMode.InheritsStream,
        test: value => value instanceof stream.Readable,
        coerce: value => new StreamReadableCoercedStream(value, StreamCoercer.DefaultOpts),
    },
    {
        mode: StreamCoercionMode.DuckPipe,
        test: value => typeof value?.pipe === 'function',
        coerce: value => new DuckPipeCoercedStream(value, StreamCoercer.DefaultOpts),
    }
];
class StreamReadableCoercedStream extends EventEmitter {
    constructor(stream, defaultOpts) {
        super();
        this.stream = stream;
        this.defaultOpts = defaultOpts;
        this.stream.on('end', () => {
            this.emit('end');
        });
    }
    ;
    getReadStream() {
        return this.stream;
    }
    pipe(outStream, opts = {}) {
        opts = { ...this.defaultOpts.pipeOptions, ...opts };
        this.stream.on('error', e => {
            this.emit('error', e);
            outStream.end();
        });
        this.stream.pipe(outStream, { end: opts.end });
    }
}
class DuckPipeCoercedStream extends EventEmitter {
    constructor(stream, defaultOpts) {
        super();
        this.stream = stream;
        this.defaultOpts = defaultOpts;
    }
    ;
    getReadStream() {
        const rs = StreamIntermediary_1.StreamIntermediary.GetStreamIntermediary();
        this.stream.pipe(rs, { end: true });
        return rs;
    }
    pipe(outStream, opts = {}) {
        opts = { ...this.defaultOpts.pipeOptions, ...opts };
        this.stream.pipe(outStream, { end: opts.end });
    }
}
//# sourceMappingURL=StreamCoercer.js.map