"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StreamIntermediary = void 0;
const stream = require("stream");
class StreamIntermediary extends stream.Duplex {
    constructor(managedApi) {
        super();
        this.managedApi = managedApi;
        this._chunks = [];
        this._done = false;
        this._readable = false;
        if (this.managedApi) {
            this.setHeader = (name, value) => {
                this.managedApi.setHeader(name, value);
            };
        }
    }
    _read(size) {
        this._readable = true;
        while (this._readable && this._chunks.length > 0) {
            const [chunk] = this._chunks.splice(0, 1);
            this._readable = this.push(chunk.chunk);
            chunk.callback(null);
        }
        if (this._done) {
            this._finalCb(null);
            return null;
        }
    }
    _write(chunk, encoding, callback) {
        this._readable = true;
        if (typeof encoding === 'string') {
            chunk = Buffer.from(chunk, encoding);
        }
        this._chunks.push({ chunk, callback });
        while (this._readable && this._chunks.length > 0) {
            const [{ chunk, callback }] = this._chunks.splice(0, 1);
            this._readable = this.push(chunk);
            callback(null);
        }
    }
    _final(callback) {
        this._done = true;
        this._finalCb = (e) => {
            if (this._readable) {
                this.push(null);
            }
            this._readable = false;
            callback(e);
        };
        if (this._chunks.length === 0) {
            this._finalCb(null);
        }
    }
    static GetStreamIntermediary(managedApi) {
        return new StreamIntermediary(managedApi);
    }
}
exports.StreamIntermediary = StreamIntermediary;
//# sourceMappingURL=StreamIntermediary.js.map