import { InternalTypeDefinition } from "./InternalTypes";
import { ClassConstructor } from '../Util/ClassConstructors';
/**
 * The symbol descriptor defines a depedency and allows us to uniquely determine
 * define a dependency for resolution.
 */
export interface ISymbolDescriptor {
    filename: string;
    symbolName: string;
    scope?: string;
}
export declare const enum DependencyInitializationTime {
    OnConstruction = "constr",
    OnUse = "use"
}
export interface IDependency {
    requirementTime: DependencyInitializationTime;
    dependency: ClassConstructor | InternalTypeDefinition;
    propertyKey: string;
}
export interface IDependencyParam extends IDependency {
    parameterIndex: number;
}
export interface IFilledDependency<T = any> extends IDependency {
    instance: T;
}
export declare class ApiDependencyCollection {
    private static readonly HashSepChar;
    private DepNo;
    private dependencyGraph;
    private registeredDependencies;
    private filledDependencies;
    instantiateDependency<C>(dependency: ClassConstructor<C>): C;
    instantiateDependency(dependency: InternalTypeDefinition): any;
    private walkDependencyGraph;
    private getFilledDependencies;
    private addFilledDependency;
    registerDependency(dep: IApiDependency): void;
    private getHash;
}
export declare const enum ApiDependencyReferenceMode {
    Constructor = "constructor",
    Function = "function"
}
export interface IApiDependency<C = {}> {
    getDependencies(): IDependency[];
    isInstantiated: boolean;
    instance: C | null;
    reference: ClassConstructor | InternalTypeDefinition;
    instantiate(dependencies: Map<ClassConstructor | InternalTypeDefinition, IApiDependency>): C;
}
export declare class ApiDependency<C = {}> implements IApiDependency<C> {
    readonly scope: string | null;
    readonly reference: ClassConstructor;
    readonly referenceMode: ApiDependencyReferenceMode;
    private readonly constructionFunc;
    private _instance;
    private constructor();
    static WithConstructor<C>(dependencyClass: ClassConstructor<C>): ApiDependency<C>;
    static WithFunc<C>(dependencyClass: ClassConstructor<C>, getterFunc: (...args: any[]) => C): ApiDependency<C>;
    getDependencies(): IDependency[];
    get isInstantiated(): boolean;
    get instance(): C | null;
    instantiate(dependencies: Map<ClassConstructor | InternalTypeDefinition, ApiDependency>): C;
}
