"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.readStreamToStringUtilCb = exports.readStreamToStringUtil = void 0;
function readStreamToStringUtil(stream, textEncoding = 'utf8') {
    let error;
    let result;
    let readPromise;
    return () => {
        if (readPromise) {
            return readPromise;
        }
        readPromise = new Promise((resolve, reject) => {
            if (error) {
                reject(error);
                return;
            }
            else if (result) {
                resolve(result);
                return;
            }
            else if (!stream.readable) {
                reject(new Error('Unreadable stream'));
                return;
            }
            const chunks = [];
            function exceptionOccured(err) {
                if (!error) {
                    error = err;
                    reject(error);
                }
            }
            stream.on('data', (chunk) => {
                if (typeof chunk === 'string' || chunk instanceof Buffer) {
                    chunks.push(chunk);
                }
                else {
                    exceptionOccured(new Error('Unexpected chunk type: ' + typeof chunk));
                }
            });
            stream.on('end', () => {
                if (!error && !result) {
                    try {
                        const chunkStrs = chunks.map(c => c instanceof Buffer
                            ? (c.toString(textEncoding))
                            : c);
                        result = chunkStrs.join('');
                        resolve(result);
                    }
                    catch (e) {
                        if (e instanceof Error) {
                            exceptionOccured(e);
                        }
                        else {
                            exceptionOccured(new Error('Unknown error occured building string: ' + e));
                        }
                    }
                }
            });
            stream.on('error', (err) => {
                exceptionOccured(err);
            });
        });
        return readPromise;
    };
}
exports.readStreamToStringUtil = readStreamToStringUtil;
function readStreamToStringUtilCb(stream, textEncoding = 'utf8') {
    return (cb) => {
        readStreamToStringUtil(stream, textEncoding)()
            .then(r => cb(undefined, r))
            .catch(e => cb(e, undefined));
    };
}
exports.readStreamToStringUtilCb = readStreamToStringUtilCb;
//# sourceMappingURL=BodyContentsUtil.js.map