import { IJsonSchema } from "openapi-types";
import { ClassConstructor } from "../Util/ClassConstructors";
export type ApiParamValidationFunction = (name: string, parsed: any) => void;
export interface __ApiParamArgsBase {
    name: string;
    typedef?: InternalTypeDefinition;
    optional?: boolean;
    description?: string;
}
type Func<T1, R> = (t1: T1) => R;
export interface __ApiParamArgsFuncs {
    initializer?: () => any;
    validationFunc?: ApiParamValidationFunction;
    regexp?: RegExp | Func<string, RegExp>;
    typeref?: ClassConstructor;
    numberMin?: number;
    numberMax?: number;
}
export type BuiltinTypeNames = 'Buffer' | 'Promise';
export interface __ApiParamArgs extends __ApiParamArgsBase, __ApiParamArgsFuncs {
}
export interface IntrinsicTypeDefinition extends IntrinsicNamedType {
    type: 'regex' | 'boolean' | 'void' | 'any';
}
export interface IntrinsicNamedType {
    typename?: string;
    uniqueTypename?: string;
}
export interface IntrinsicTypeDefinitionString extends IntrinsicNamedType {
    type: 'string';
    validationRegex?: RegExp;
    schema?: {
        enum?: string[];
    } | {
        const?: string;
    };
    typename?: string;
}
export interface IntrinsicTypeDefinitionNumber extends IntrinsicNamedType {
    type: 'number';
    schema?: {
        enum?: number[];
    } | {
        const?: number;
    };
    typename?: string;
}
export type InternalTypeDefinition = IntrinsicTypeDefinition | IntrinsicTypeDefinitionString | IntrinsicTypeDefinitionNumber | InternalObjectTypeDefinition | InternalExternalSchemaTypeDefinition | InternalBuiltinObjectTypeDefinition | InternalUnionTypeDefinition | InternalIntersectionTypeDefinition | InternalArrayTypeDefinition | InternalDateTypeDefinition | InternalFunctionTypeDefinition | InternalEnumTypeDefinition;
export interface InternalFunctionTypeDefinition extends IntrinsicNamedType {
    type: 'function';
}
export interface InternalDateTypeDefinition extends IntrinsicNamedType {
    type: 'date';
}
export interface InternalObjectTypeDefinition extends IntrinsicNamedType {
    type: 'object';
    schema?: IJsonSchemaWithRefs;
}
type WithRequired<T, K extends keyof T> = T & {
    [P in K]-?: T[P];
};
export interface InternalExternalSchemaTypeDefinition extends IntrinsicNamedType {
    type: 'external';
    schema: WithRequired<IJsonSchemaWithRefs, '$schema'>;
}
export interface InternalEnumTypeDefinition extends IntrinsicNamedType {
    type: 'enum';
    schema?: {
        enum?: Array<string | number>;
    };
    typename?: string;
}
export interface InternalBuiltinObjectTypeDefinition extends IntrinsicNamedType {
    type: BuiltinTypeNames;
}
export interface InternalUnionTypeDefinition extends IntrinsicNamedType {
    type: 'union';
    types: InternalTypeDefinition[];
}
export interface InternalArrayTypeDefinition extends IntrinsicNamedType {
    type: 'array';
    elementType: InternalTypeDefinition;
}
export interface InternalIntersectionTypeDefinition extends IntrinsicNamedType {
    type: 'intersection';
    types: InternalTypeDefinition[];
}
export interface IJsonSchemaWithRefs extends IJsonSchema {
    $ref?: string;
}
export declare abstract class InternalTypeUtil {
    static readonly TypeString: IntrinsicTypeDefinitionString;
    static readonly TypeNumber: IntrinsicTypeDefinitionNumber;
    static readonly TypeEnum: InternalEnumTypeDefinition;
    static readonly TypeRegex: IntrinsicTypeDefinition;
    static readonly TypeDate: InternalDateTypeDefinition;
    static readonly TypeBoolean: IntrinsicTypeDefinition;
    static readonly TypeAnyObject: InternalObjectTypeDefinition;
    static readonly TypeAnyArray: InternalArrayTypeDefinition;
    static readonly TypeNumberArray: InternalArrayTypeDefinition;
    static readonly TypeAnyFunction: InternalFunctionTypeDefinition;
    static readonly TypeAny: IntrinsicTypeDefinition;
    static readonly TypeBuffer: InternalBuiltinObjectTypeDefinition;
}
export {};
