import { IApiDefinition, ApiMethod, IApiParamDefinition, IApiProcessors, IApiModifierDefinition } from "./ApiDefinition";
import 'reflect-metadata';
import { IDependency, IDependencyParam } from "./ApiDependency";
import { ClassConstructor } from "../Util/ClassConstructors";
import { IApiProcessor, IApiGlobalProcessor } from "./ApiProcessing/ApiProcessing";
import { IApiInvocationParams, IApiInvocationResult } from "./ManagedApi";
export interface IApiClassDefinition {
    constructor: ClassConstructor;
    handlers: Map<ApiMethod, Map<string, IApiDefinition>>;
}
export declare class ManagedApiInternal {
    static readonly ApiMetadataKey = "managedapi:api";
    static readonly ApiMethodMetadataKey = "managedapi:apimethod";
    static readonly ApiMethodModifierMetadataKey = "managedapi:apimodifier";
    static readonly ApiMethodProcessorsMetadataKey = "managedapi:apiprocessors";
    static readonly ApiMethodParamsMetadataKey = "managedapi:apimethodparams";
    static readonly DependencyMetadataKey = "managedapi:dependency";
    static readonly DependencyParamMetadataKey = "managedapi:dependencyparams";
    static readonly GlobalApiProcessorMetadataKey = "managedapi:globalapiprocessor";
    static ResetRegisteredApis(): void;
    static RegisterApi(constructor: ClassConstructor): void;
    static GetRegisteredApiClassDefinitions(): IApiClassDefinition[];
    static GetHandlersForConstructor(constructor: ClassConstructor): Map<ApiMethod, Map<string, IApiDefinition>>;
    static GetDependenciesOnConstructor(constructor: ClassConstructor): IDependency[];
    static ErrorMultipleApiDefinition(method: ApiMethod, route: string): Error;
    static AddApiMetadataToObject(metadata: IApiDefinition, target: object): void;
    static AddApiModifierMetadataToObject<T = object>(metadata: IApiModifierDefinition<T>, target: object): void;
    static AddGlobalApiProcessorMetadataToObject(metadata: IApiGlobalProcessor, target: object): void;
    static AddApiProcessorsToObject(processors: IApiProcessor<IApiInvocationParams<any> | IApiInvocationResult>[], target: object): void;
    static AddApiHandlerParamMetadataToObject(metadata: IApiParamDefinition, target: object): void;
    static AddApiDependencyParamMetadataToObject(metadata: IDependencyParam, target: object): void;
    static AddDependencyMetadataToObject(metadata: IDependency, target: object): void;
    static GetDependencyParams(target: object, key: string | symbol): IDependencyParam[];
    static GetApiHandlerParams(target: object, key: string | symbol): IApiParamDefinition[];
    static GetApiHandlerProcessors<T extends object>(target: object): IApiProcessors<T>;
    private static GetApiDefinitionsOnObject;
    static GetApiModifierDefinitionsOnObject<T = object>(target: object, propertyKey: string | symbol): IApiModifierDefinition<T>[];
    static GetGlobalApiProcessorOnObject(target: object): IApiGlobalProcessor[];
    static GetApiProcessorsOnObject(target: object): IApiProcessor<IApiInvocationParams<any> | IApiInvocationResult>[];
    static GetHandlerParamDefinitionsOnObject(target: object, key: string | symbol): IApiParamDefinition[];
    static GetDependencyDefinitionsOnObject(target: object): IDependency[];
    static GetDependencyParamDefinitionsOnObject(target: object, key: string | symbol): IDependencyParam[];
}
