"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiMimeType = exports.parseApiMimeType = void 0;
const GeneratedErrors_1 = require("../GeneratedErrors");
function parseApiMimeType(contentTypeHeader, defaultCharset = 'latin1') {
    const parts = contentTypeHeader ? contentTypeHeader.toLowerCase().split(';') : [];
    const header = {
        mimeType: parts[0] || ApiMimeType.ApplicationOctetStream,
        charset: defaultCharset,
    };
    // Parse mime type
    switch (parts[0]) {
        case 'application/json':
            header.mimeType = ApiMimeType.ApplicationJson;
            break;
        case 'application/javascript':
            header.mimeType = ApiMimeType.ApplicationJavascript;
            break;
        case 'application/xml':
            header.mimeType = ApiMimeType.ApplicationXml;
            break;
        case 'application/octet-stream':
            header.mimeType = ApiMimeType.ApplicationOctetStream;
            break;
        case 'text':
        case 'text/plain':
        case 'application/text':
            header.mimeType = ApiMimeType.Text;
            break;
    }
    // parse args
    for (let i = 1; i < parts.length; ++i) {
        const [argType, arg] = parts[i].split('=', 2);
        switch (argType.trim()) {
            case 'charset':
                header.charset = normalizeCharset(arg);
                break;
            case 'boundary':
                header.boundary = arg;
                break;
        }
    }
    return header;
}
exports.parseApiMimeType = parseApiMimeType;
function normalizeCharset(charset) {
    charset = charset.toLowerCase().trim();
    switch (charset) {
        case 'utf-8':
        case 'utf8':
            return 'utf8';
        case 'utf16le':
        case 'utf-16le':
            return 'utf16le';
        case 'latin1':
        case 'latin-1':
        case 'iso-8859-1':
            return 'latin1';
        case 'ucs-2':
        case 'ucs2':
            return 'ucs2';
        case 'ascii':
        case 'base64':
        case 'bindary':
        case 'hex':
            return charset;
    }
    throw new GeneratedErrors_1.HttpUnsupportedMediaTypeError('Charset not supported');
}
var ApiMimeType;
(function (ApiMimeType) {
    ApiMimeType["ApplicationJson"] = "application/json";
    ApiMimeType["ApplicationJavascript"] = "application/javascript";
    ApiMimeType["ApplicationXml"] = "application/xml";
    ApiMimeType["ApplicationOctetStream"] = "application/octet-stream";
    ApiMimeType["Text"] = "text/plain";
    ApiMimeType["TextXml"] = "text/xml";
    ApiMimeType["MultipartFormData"] = "multipart/form-data";
})(ApiMimeType || (exports.ApiMimeType = ApiMimeType = {}));
//# sourceMappingURL=MimeTypes.js.map