"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CliCommand = void 0;
const HandlerTree_1 = require("../transformer/HandlerTree");
class CliCommand {
    constructor() {
        this.console = console;
    }
    async wrapCliError(f) {
        try {
            await f();
        }
        catch (e) {
            this.handleError(e);
            throw e;
        }
    }
    async handleError(err) {
        console.error('Error processing command: ');
        console.error(err);
    }
    disableConsoleOutput() {
        this.console = { ...console };
        for (const key of Object.keys(console)) {
            if (typeof console[key] === 'function') {
                console[key] = () => { };
            }
        }
    }
    enableConsoleOutput() {
        for (const key of Object.keys(this.console)) {
            if (typeof console[key] === 'function') {
                console[key] = this.console[key];
            }
        }
    }
    printExtractionSummary(options, api, tree) {
        const Table = require('cli-table');
        const table = new Table({
            head: ['Method', 'Route', 'File'],
        });
        const methodRoutes = new Map();
        for (const handler of (0, HandlerTree_1.WalkTreeByType)(tree, HandlerTree_1.isHandlerNode)) {
            if (!methodRoutes.has(handler.apiMethod)) {
                methodRoutes.set(handler.apiMethod, new Set());
            }
            const methodMath = methodRoutes.get(handler.apiMethod);
            if (!methodMath.has(handler.route)) {
                methodMath.add(handler.route);
                table.push([handler.apiMethod, handler.route, handler.location.file]);
            }
        }
        ;
        return table.toString();
    }
}
exports.CliCommand = CliCommand;
//# sourceMappingURL=CliCommand.js.map