import { IExtractor } from "./IExtractor";
import { OpenAPIV3 } from 'openapi-types';
import * as yaml from 'js-yaml';
import { IProgramInfo } from "./IProgramInfo";
import { IHandlerTreeNodeRoot, IHandlerTreeNodeParameter } from "../transformer/HandlerTree";
export interface IOpenApiV3Opts {
    disableTryInferSchemes?: boolean;
    /**
     * additionalProperties defaults to `true` so it shouldn't need to be outputted manually
     * and may cause problems with some tools (e.g. autorest)
     */
    outputAdditionalPropertiesTrue?: boolean;
    yamlOpts?: yaml.DumpOptions;
}
export declare class OpenApiV3Extractor implements IExtractor {
    private readonly apiTree;
    private readonly apiInfo;
    private readonly opts;
    static readonly SwaggerVersion = "3.0.1";
    private static readonly RouteSeperator;
    private static readonly MimeTypeText;
    private static readonly MimeTypeJson;
    private static readonly ExcludeMetadataTags;
    private readonly removableTypes;
    private tags;
    private definitions;
    private definitionNameMap;
    constructor(apiTree: IHandlerTreeNodeRoot, apiInfo: IProgramInfo, opts: IOpenApiV3Opts);
    getDocument(): OpenAPIV3.Document;
    private getApiServers;
    private getComponents;
    private tryInferSchemes;
    private validHost;
    private getPaths;
    private swaggerizeRoute;
    private getOperationObject;
    private isBodyParam;
    private getResponseObject;
    private getOperationResponseDescription;
    private getInlineTypeSchema;
    private getReturnTypeEnumValue;
    private getEnumTypeInline;
    private getInternalSchemaToOutputSchema;
    private expectedDefinitions;
    private addDefinitions;
    private renameDefinition;
    private fixDefinitionProperties;
    private _fixDefinitionPropertiesCollection;
    private fixDefinitionRefs;
    private fixArrayDefinitionRefs;
    private replaceRefStr;
    private recordTagObject;
    private getTagObject;
    private getParametersObject;
    private isParameterRequired;
    getRequestBody(p: IHandlerTreeNodeParameter[]): OpenAPIV3.ReferenceObject | OpenAPIV3.RequestBodyObject;
    private getMultipartRequestBodySchema;
    private getSingleRequestBody;
    toString(): string;
}
export declare class OpenApiV3JsonExtractor extends OpenApiV3Extractor {
    toString(): string;
}
