"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TsApiCommandLine = void 0;
const commander = require("commander");
const path = require("path");
const CommandUtil_1 = require("./CommandUtil");
const ExtractCommand_1 = require("./ExtractCommand");
class TsApiCommandLine {
    constructor() {
        this.commands = new Set();
        this.program = new commander.Command();
        this.program.version((0, CommandUtil_1.getPackageVersion)());
    }
    addComamnd(constr) {
        const cmd = new constr(this.program);
        this.commands.add(cmd);
        return cmd;
    }
    addCommands(commands) {
        return commands.map(c => this.addComamnd(c));
    }
    registerCommand(cmd) {
        this.commands.add(cmd);
        cmd.registerWithProgram(this.program);
    }
    getBuiltinCommands() {
        return [
            ExtractCommand_1.ExtractCommand,
            ...this.tryGetPeerLibCommands(),
        ];
    }
    tryGetPeerLibCommands() {
        const peerLibs = ['azure-function', 'express'];
        const paths = [
            path.join(__dirname, '../../../'),
            path.join(__dirname, '../../'),
            path.join(__dirname, '../'),
            path.join(__dirname, './'),
        ];
        let outLibs = [];
        for (const libName of peerLibs) {
            let libPath;
            try {
                libPath = require.resolve(`ts-api-decorators-${libName}/dist/commands`, { paths });
            }
            catch (e) {
                continue;
            }
            const lib = require(libPath);
            if (Array.isArray(lib.default)) {
                outLibs = outLibs.concat(lib.default);
            }
        }
        return outLibs;
    }
    parse(argv) {
        this.program.parse(argv);
    }
}
exports.TsApiCommandLine = TsApiCommandLine;
//# sourceMappingURL=TsApiCommandLine.js.map