import 'reflect-metadata';
import { ApiMethodCallbackFunction } from "../apiManagement/ApiDefinition";
import { HandlerMethodDecorator } from "../transformer/HandlerMethodDecorator";
import { ClassDecorator } from "../transformer/ClassDecorator";
import { ClassConstructor } from '../Util/ClassConstructors';
export type ApiMethodDecoratorReturnType<T, K = (...args: any[]) => T> = (target: object, propertyKey: string, descriptor: TypedPropertyDescriptor<K>) => void;
declare abstract class ApiClassDecorators {
    static Api<T extends ClassConstructor>(constructor: T): void;
}
export declare const Api: typeof ApiClassDecorators.Api;
declare abstract class ApiMethodDecorators {
    static ApiGetMethod<T extends string>(route: string, responseCodes?: number[]): ApiMethodDecoratorReturnType<T | Promise<T>>;
    static ApiGetMethod<T extends void, K extends (string | object)>(route: string, responseCodes?: number[]): ApiMethodDecoratorReturnType<T, (callback: ApiMethodCallbackFunction<K>, ...args: any[]) => T>;
    static ApiGetMethod<T extends object>(route: string, responseCodes?: number[]): ApiMethodDecoratorReturnType<T | Promise<T>>;
    static ApiGetMethod<T extends void>(route: string, responseCodes?: number[]): ApiMethodDecoratorReturnType<void | Promise<void>>;
    static ApiGetSchemaMethod<T extends object>(route: string): ApiMethodDecoratorReturnType<any | Promise<any>>;
    static ApiPostMethod<T extends string>(route: string, responseCodes?: number[]): ApiMethodDecoratorReturnType<T | Promise<T>>;
    static ApiPostMethod<T extends void, K extends (string | object)>(route: string, responseCodes?: number[]): ApiMethodDecoratorReturnType<T, (callback: ApiMethodCallbackFunction<K>, ...args: any[]) => T>;
    static ApiPostMethod<T extends object>(route: string, responseCodes?: number[]): ApiMethodDecoratorReturnType<T | Promise<T>>;
    static ApiPostMethod<T extends void>(route: string, responseCodes?: number[]): ApiMethodDecoratorReturnType<void | Promise<void>>;
    static ApiPutMethod<T extends string>(route: string, responseCodes?: number[]): ApiMethodDecoratorReturnType<T | Promise<T>>;
    static ApiPutMethod<T extends void, K extends (string | object)>(route: string, responseCodes?: number[]): ApiMethodDecoratorReturnType<T, (callback: ApiMethodCallbackFunction<K>, ...args: any[]) => T>;
    static ApiPutMethod<T extends object>(route: string, responseCodes?: number[]): ApiMethodDecoratorReturnType<T | Promise<T>>;
    static ApiPutMethod<T extends void>(route: string, responseCodes?: number[]): ApiMethodDecoratorReturnType<void | Promise<void>>;
    static ApiDeleteMethod<T extends string>(route: string, responseCodes?: number[]): ApiMethodDecoratorReturnType<T | Promise<T>>;
    static ApiDeleteMethod<T extends void, K extends (string | object)>(route: string, responseCodes?: number[]): ApiMethodDecoratorReturnType<T, (callback: ApiMethodCallbackFunction<K>, ...args: any[]) => T>;
    static ApiDeleteMethod<T extends object>(route: string, responseCodes?: number[]): ApiMethodDecoratorReturnType<T | Promise<T>>;
    static ApiDeleteMethod<T extends void>(route: string, responseCodes?: number[]): ApiMethodDecoratorReturnType<void | Promise<void>>;
    private static wrapApiMethod;
    private static wrapApiMethodWithHandler;
    private static toResponseCodesArray;
}
export declare const GetApiDecorator: (param: string | Function) => ClassDecorator;
export declare const GetApiMethodDecorator: (param: string | Function) => HandlerMethodDecorator;
export declare const ApiGetMethod: typeof ApiMethodDecorators.ApiGetMethod;
export declare const ApiPutMethod: typeof ApiMethodDecorators.ApiPutMethod;
export declare const ApiPostMethod: typeof ApiMethodDecorators.ApiPostMethod;
export declare const ApiDeleteMethod: typeof ApiMethodDecorators.ApiDeleteMethod;
export declare const ApiGetSchemaMethod: typeof ApiMethodDecorators.ApiGetSchemaMethod;
export {};
