import { ApiParamValidationFunction } from '../apiManagement/InternalTypes';
import { HandlerMethodParameterDecorator } from "../transformer/HandlerMethodParameterDecorator";
declare abstract class BodyParams {
    /**
     * Decorates a body parameter that should be validated with a regular expression.
     * @param stringValidationRegex The regular expression to validate the input
     */
    static ApiBodyParamString(stringValidationRegex?: RegExp): any;
    /**
     * Decorates a body parameter that should be cast to a number.
     * @param numberMin The minimum value, undefined for no minimum.
     * @param numberMax The maximum value, undefined for no maximum.
     * @param numberDefault The default value, undefined will use the minimum value if defined, if not the maximum, if not then undefined.
     */
    static ApiBodyParamNumber(numberMin?: number, numberMax?: number): any;
    /**
     * Decorates a body parameter that receives a raw, readable stream for the request
     * @param mimeType The valid mime type for the request.
     */
    static ApiBodyParamStream(mimeType?: string): any;
    static ApiBodyParamMultipartFormFileName(stringValidationRegex?: RegExp): any;
    static ApiBodyParamMultipartFormFileString(mimeType?: string): any;
    static ApiBodyParamMultipartFormFileStream(mimeType?: string): any;
    /**
     * Decorates a body parameter that receives an unparsed string representing the request body.
     * @param mimeType The valid mime type for the request.
     */
    static ApiBodyParamRawString(mimeType?: string): any;
    /**
     * The request body.
     * @param validator A validation function to validate the body contents
     */
    static ApiBodyParam(): ParameterDecorator;
    static ApiBodyParam(validator?: ApiParamValidationFunction): ParameterDecorator;
}
export declare const GetBodyParamDecorator: (param: string | Function) => HandlerMethodParameterDecorator;
export declare const ApiBodyParam: typeof BodyParams.ApiBodyParam;
export declare const ApiBodyParamString: typeof BodyParams.ApiBodyParamString;
export declare const ApiBodyParamNumber: typeof BodyParams.ApiBodyParamNumber;
export declare const ApiBodyParamStream: typeof BodyParams.ApiBodyParamStream;
export declare const ApiBodyParamRawString: typeof BodyParams.ApiBodyParamRawString;
export {};
