import { ApiParamValidationFunction } from "../apiManagement/InternalTypes";
import { HandlerMethodParameterDecorator } from "../transformer/HandlerMethodParameterDecorator";
declare abstract class HeaderParams {
    /**
     * Decorates a query parameter that should be validated with a regular expression.
     * @param stringValidationRegex The regular expression to validate the input
     */
    static ApiHeaderParamString(paramName?: string, stringValidationRegex?: RegExp): any;
    /**
     * Decorates a query parameter that should be cast to a number.
     * @param numberMin The minimum value, undefined for no minimum.
     * @param numberMax The maximum value, undefined for no maximum.
     * @param numberDefault The default value, undefined will use the minimum value if defined, if not the maximum, if not then undefined.
     */
    static ApiHeaderParamNumber(paramName?: string, numberMin?: number, numberMax?: number): any;
    /**
     * A query parameter.
     * @param validator
     */
    static ApiHeaderParam(paramName?: string): ParameterDecorator;
    static ApiHeaderParam(paramName?: string, validator?: ApiParamValidationFunction): ParameterDecorator;
}
export declare const GetHeaderParamDecorator: (param: string | Function) => HandlerMethodParameterDecorator;
export declare const ApiHeaderParam: typeof HeaderParams.ApiHeaderParam;
export declare const ApiHeaderParamString: typeof HeaderParams.ApiHeaderParamString;
export declare const ApiHeaderParamNumber: typeof HeaderParams.ApiHeaderParamNumber;
export {};
