import { ApiParamValidationFunction } from "../apiManagement/InternalTypes";
import { HandlerMethodParameterDecorator } from "../transformer/HandlerMethodParameterDecorator";
declare abstract class PathParams {
    /**
     * Decorates a path parameter that should be validated with a regular expression.
     * @param stringValidationRegex The regular expression to validate the input
     */
    static ApiPathParamString(paramName?: string, stringValidationRegex?: RegExp): any;
    /**
     * Decorates a path parameter that should be cast to a number.
     * @param numberMin The minimum value, undefined for no minimum.
     * @param numberMax The maximum value, undefined for no maximum.
     * @param numberDefault The default value, undefined will use the minimum value if defined, if not the maximum, if not then undefined.
     */
    static ApiPathParamNumber(paramName?: string, numberMin?: number, numberMax?: number): any;
    /**
     * A path parameter.
     * @param validator
     */
    static ApiPathParam(paramName?: string): ParameterDecorator;
    static ApiPathParam(paramName: string, validator?: ApiParamValidationFunction): ParameterDecorator;
}
export declare const GetPathParamDecorator: (param: string | Function) => HandlerMethodParameterDecorator;
export declare const ApiPathParam: typeof PathParams.ApiPathParam;
export declare const ApiPathParamString: typeof PathParams.ApiPathParamString;
export declare const ApiPathParamNumber: typeof PathParams.ApiPathParamNumber;
export {};
