import { ApiParamValidationFunction } from "../apiManagement/InternalTypes";
import { HandlerMethodParameterDecorator } from "../transformer/HandlerMethodParameterDecorator";
declare abstract class QueryParams {
    /**
     * Decorates a query parameter that should be validated with a regular expression.
     * @param stringValidationRegex The regular expression to validate the input
     */
    static ApiQueryParamString(paramName?: string, stringValidationRegex?: RegExp): any;
    /**
     * Decorates a query parameter that should be cast to a number.
     * @param numberMin The minimum value, undefined for no minimum.
     * @param numberMax The maximum value, undefined for no maximum.
     */
    static ApiQueryParamNumber(paramName?: string, numberMin?: number, numberMax?: number): any;
    /**
     * A query parameter.
     * @param validator
     */
    static ApiQueryParam(paramName?: string): ParameterDecorator;
    static ApiQueryParam(paramName: string, validator?: ApiParamValidationFunction): ParameterDecorator;
}
export declare const GetQueryParamDecorator: (param: string | Function) => HandlerMethodParameterDecorator;
export declare const ApiQueryParam: typeof QueryParams.ApiQueryParam;
export declare const ApiQueryParamString: typeof QueryParams.ApiQueryParamString;
export declare const ApiQueryParamNumber: typeof QueryParams.ApiQueryParamNumber;
export {};
