"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClassPropertyDecorator = void 0;
const ts = require("typescript");
const DecoratorDefinition_1 = require("./DecoratorDefinition");
const HandlerTree_1 = require("./HandlerTree");
const Decorator_1 = require("./Decorator");
const TransformerMetadata_1 = require("./TransformerMetadata");
const ExpressionWrapper_1 = require("./ExpressionWrapper");
const CompilationError_1 = require("../Util/CompilationError");
class ClassPropertyDecorator extends Decorator_1.Decorator {
    constructor(definition) {
        super({
            ...definition,
            decoratorType: DecoratorDefinition_1.DecoratorType.ClassProperty,
        }, Decorator_1.DecoratorNodeType.ClassProperty);
    }
    get memberTypeRestrictions() {
        return this.definition.memberTypeRestrictions;
    }
    getDecoratorTreeElement(parent, node, decorator, context) {
        let argumentResult = this.applyArguments(node, decorator, context);
        return {
            transformedDecorator: argumentResult.decorator,
            decoratorTreeNode: {
                type: HandlerTree_1.HandlerTreeNodeType.DependencyProperty,
                decorator: this.definition,
                children: [],
                location: this.getNodeLocation(node),
                parent,
                metadata: argumentResult.metadata,
            }
        };
    }
    *getDefaultMetadataGetters() {
        yield* super.getDefaultMetadataGetters();
        yield (node, decorator, context) => this.getNodeTyperefMetadata(node, decorator, context);
        yield (node, decorator, context) => this.getNodeTypedefMetadata(node, decorator, context);
        yield (node, decorator, context) => this.getNodeOptionalMetadata(node, decorator, context);
        yield (node, decorator, context) => this.getNodeInitializerMetadata(node, decorator, context);
    }
    getNodeTyperefMetadata(node, decorator, context) {
        let internalType = {
            type: 'any',
        };
        if (node.type) {
            const type = context.typeChecker.getTypeFromTypeNode(node.type);
            internalType = context.typeSerializer.getInternalTypeRepresentation(node.type, type);
            if (this.memberTypeRestrictions
                && !this.memberTypeRestrictions.find(t => t.type === internalType.type || t.type === 'any')) {
                throw new CompilationError_1.CompilationError('Invalid type for decorator: ' + internalType.type, node);
            }
        }
        return {
            ...TransformerMetadata_1.BuiltinMetadata.Typedef,
            value: internalType,
        };
    }
    getNodeTypedefMetadata(node, decorator, context) {
        if (node.type) {
            const type = context.typeChecker.getTypeFromTypeNode(node.type);
            let internalType = context.typeSerializer.getInternalTypeRepresentation(node.type, type);
            if (this.memberTypeRestrictions
                && !this.memberTypeRestrictions.find(t => t.type === internalType.type || t.type === 'any')) {
                throw new CompilationError_1.CompilationError('Invalid type for decorator: ' + internalType.type, node);
            }
            if (internalType.type === 'object' && type.isClass() && ts.isTypeReferenceNode(node.type)) {
                if (ts.isIdentifier(node.type.typeName)) {
                    return {
                        ...TransformerMetadata_1.BuiltinMetadata.Typeref,
                        value: new ExpressionWrapper_1.ExpressionWrapper(node.type.typeName),
                    };
                }
            }
        }
    }
    getNodeOptionalMetadata(node, decorator, context) {
        let optional = !!node.questionToken;
        if (optional) {
            return {
                ...TransformerMetadata_1.BuiltinMetadata.Optional,
                value: optional,
            };
        }
    }
    getNodeInitializerMetadata(node, decorator, context) {
        if (node.initializer) {
            let parenExpr = this.parenthesizeExpression(node.initializer);
            return {
                ...TransformerMetadata_1.BuiltinMetadata.Initializer,
                value: new ExpressionWrapper_1.ExpressionWrapper(ts.factory.createArrowFunction(undefined, undefined, [], undefined, undefined, parenExpr)),
            };
        }
    }
}
exports.ClassPropertyDecorator = ClassPropertyDecorator;
//# sourceMappingURL=ClassPropertyDecorator.js.map