import * as ts from 'typescript';
import { IDecoratorDefinitionBase, IDecoratorArgument, DecoratorNodeTreeHierarchyType } from './DecoratorDefinition';
import { ITransformedTreeElement, IHandlerLocation, IHandlerTreeNode } from './HandlerTree';
import { ITransformContext } from './ITransformContext';
import { ITransformerMetadata } from './TransformerMetadata';
export declare enum DecoratorNodeType {
    Class = 0,
    ClassProperty = 1,
    Method = 2,
    Parameter = 3
}
export interface IDecorator<N extends ts.Node = ts.Node> extends IDecoratorDefinitionBase {
    nodeType: DecoratorNodeType;
    treeHierarchyType: DecoratorNodeTreeHierarchyType;
    isSourceFileMatch(sourceFile: ts.SourceFile): boolean;
    getDecoratorTreeElement(parent: IHandlerTreeNode | undefined, node: N, decorator: ts.Decorator, context: ITransformContext): ITransformedTreeElement<ts.Decorator>;
}
interface IExprWithMetadata {
    expression?: ts.Expression;
    metadata?: ITransformerMetadata;
}
export interface IIndexTsInfo {
    dirnameWithFile: string;
    extname: string;
}
export declare abstract class Decorator<N extends ts.Node, DT extends IDecoratorDefinitionBase> implements IDecorator<N> {
    protected definition: DT;
    readonly nodeType: DecoratorNodeType;
    private readonly _indexTs;
    get magicFunctionName(): string;
    get indexTs(): string;
    get provider(): string;
    get decoratorType(): DT['decoratorType'];
    get dependencies(): (import("./DecoratorDefinition").IDecoratorDependency | import("../Util/Func").Func1<IHandlerTreeNode, void | import("./DecoratorDefinition").IDecoratorDependency[]>)[];
    get arguments(): IDecoratorArgument[];
    get metadata(): ITransformerMetadata[];
    get transformArgumentsToObject(): boolean | import("./TransformerMetadata").IMetadataDescriptor[];
    get isCallExpression(): boolean;
    get treeHierarchyType(): DecoratorNodeTreeHierarchyType;
    constructor(definition: DT, nodeType: DecoratorNodeType);
    private getPathWithoutExt;
    private getPathWithoutCommonRoot;
    private getPathFromCommonRoot;
    private getExt;
    abstract getDecoratorTreeElement(parent: IHandlerTreeNode | undefined, node: N, decorator: ts.Decorator, context: ITransformContext): ITransformedTreeElement<ts.Decorator>;
    protected getNodeLocation(node: N): IHandlerLocation;
    protected getMetadata(node: N, decorator: ts.Decorator, context: ITransformContext): ITransformerMetadata[];
    protected getDefaultMetadataGetters(): Generator<(node: N, decorator: ts.Decorator, context: ITransformContext) => ITransformerMetadata, void, unknown>;
    protected getNodeNameMetadata(node: N): ITransformerMetadata;
    protected getDecoratorTypeArgumentsMetadata(node: N, decorator: ts.Decorator): ITransformerMetadata;
    protected applyArguments(node: N, decorator: ts.Decorator, context: ITransformContext): {
        metadata: ITransformerMetadata[];
        decorator: ts.Decorator;
    };
    private getUnparenthesizedExpression;
    private checkTypeCompatibility;
    protected getOutputDecoratorArgs(expression: ts.CallExpression, exprArguments: IExprWithMetadata[], metadataCollection: ITransformerMetadata[], context: ITransformContext): ts.CallExpression;
    protected parenthesizeExpression(expression: ts.Expression): ts.ParenthesizedExpression;
    isSourceFileMatch(sourceFile: ts.SourceFile): boolean;
}
export {};
