import * as ts from 'typescript';
import * as tjs from "typescript-json-schema";
import { ITreeTransformer } from "./ITreeTransformer";
import { __ApiParamArgsFuncs, __ApiParamArgsBase } from '../apiManagement/InternalTypes';
import { TypeSerializer } from './TypeSerializer';
export interface IDecorationFunctionTransformInfoBase {
    magicFunctionName: string;
    indexTs: string;
}
export declare type TypePredicateFunc<P, T extends P> = (p: P) => p is T;
export interface IDecoratorFunctionTransformInfo<T extends ts.Node> {
    nodeCheckFunction: TypePredicateFunc<ts.Node, T>;
}
export declare class ExpressionWrapper {
    node: ts.Expression;
    constructor(node: ts.Expression);
}
export declare type ExpressionWrappedType<T extends object> = {
    [P in keyof T]?: T[P] | ExpressionWrapper;
};
export declare type ParamArgsInitializer = __ApiParamArgsBase & ExpressionWrappedType<__ApiParamArgsFuncs>;
export declare abstract class DecoratorTransformer<T extends ts.Node, I extends IDecorationFunctionTransformInfoBase = IDecorationFunctionTransformInfoBase> implements ITreeTransformer {
    protected readonly program: ts.Program;
    protected readonly generator: tjs.JsonSchemaGenerator;
    protected readonly transformInfo: I & IDecoratorFunctionTransformInfo<T>;
    protected readonly typeChecker: ts.TypeChecker;
    protected readonly typeSerializer: TypeSerializer;
    constructor(program: ts.Program, generator: tjs.JsonSchemaGenerator, transformInfo: I & IDecoratorFunctionTransformInfo<T>);
    visitNodeAndChildren(node: ts.SourceFile, context: ts.TransformationContext): ts.SourceFile;
    visitNodeAndChildren(node: ts.Node, context: ts.TransformationContext): ts.Node;
    abstract visitNode(node: T): T;
    private __visitNode;
    protected isArgumentDecoratorCallExpression(decorator: ts.Decorator['expression'], transformInfo?: IDecorationFunctionTransformInfoBase): decorator is ts.CallExpression;
    protected isDecoratedParameterExpression(node: ts.Node, transformInfo?: IDecorationFunctionTransformInfoBase): node is ts.ParameterDeclaration;
    protected parenthesizeExpression(expression: ts.Expression): ts.ParenthesizedExpression;
}
