"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const path = require("path");
const TypeSerializer_1 = require("./TypeSerializer");
class ExpressionWrapper {
    constructor(node) {
        this.node = node;
    }
}
exports.ExpressionWrapper = ExpressionWrapper;
class DecoratorTransformer {
    constructor(program, generator, transformInfo) {
        this.program = program;
        this.generator = generator;
        this.transformInfo = transformInfo;
        this.typeChecker = this.program.getTypeChecker();
        this.typeSerializer = new TypeSerializer_1.TypeSerializer(program, generator, this.typeChecker);
    }
    visitNodeAndChildren(node, context) {
        return ts.visitEachChild(this.__visitNode(node), childNode => this.visitNodeAndChildren(childNode, context), context);
    }
    __visitNode(node) {
        if (!this.transformInfo.nodeCheckFunction(node)) {
            return node;
        }
        return this.visitNode(node);
    }
    isArgumentDecoratorCallExpression(decorator, transformInfo = this.transformInfo) {
        if (!ts.isCallExpression(decorator)) {
            return false;
        }
        const signature = this.typeChecker.getResolvedSignature(decorator);
        if (typeof signature === 'undefined') {
            return false;
        }
        const { declaration } = signature;
        if (!(!!declaration
            && !ts.isJSDocSignature(declaration)
            && !!declaration.name
            && declaration.name.getText() === transformInfo.magicFunctionName)) {
            return false;
        }
        const sourceFile = path.join(declaration.getSourceFile().fileName);
        return sourceFile.endsWith(transformInfo.indexTs + '.ts') || sourceFile.endsWith(transformInfo.indexTs + '.d.ts');
    }
    isDecoratedParameterExpression(node, transformInfo = this.transformInfo) {
        if (!ts.isParameter(node)) {
            return false;
        }
        if (!node.decorators || node.decorators.length === 0) {
            return false;
        }
        for (const decorator of node.decorators) {
            if (this.isArgumentDecoratorCallExpression(decorator.expression)) {
                return true;
            }
        }
        return false;
    }
    parenthesizeExpression(expression) {
        return ts.isParenthesizedExpression(expression)
            ? expression
            : ts.factory.createParenthesizedExpression(expression);
    }
}
exports.DecoratorTransformer = DecoratorTransformer;
//# sourceMappingURL=DecoratorTransformer.js.map