import * as ts from 'typescript';
import * as tjs from "typescript-json-schema";
import { DecoratorTransformer, IDecorationFunctionTransformInfoBase } from './DecoratorTransformer';
import { ParamDecoratorTransformerInfo } from './ParamDecoratorTransformer';
import { IApiDefinitionBase, IApiParamDefinition } from '../apiManagement/ApiDefinition';
import { ITransformerMetadataCollection } from './TransformerMetadata';
import { IMetadataResolver } from './MetadataManager';
import { IApiMethodDecoratorDefinition } from '..';
export interface IExtractedApiDefinition extends IApiDefinitionBase {
    file: string;
    parameters: IApiParamDefinition[];
}
export interface IExtractedApiDefinitionWithMetadata extends IExtractedApiDefinition, ITransformerMetadataCollection {
}
export interface IExtractedTag {
    name: string;
    description?: string;
}
export declare type OnApiMethodExtractedHandler = (method: IExtractedApiDefinitionWithMetadata) => void;
export interface IExtractionTransformInfoBase extends IDecorationFunctionTransformInfoBase, IApiMethodDecoratorDefinition {
    parameterTypes: ParamDecoratorTransformerInfo[];
}
interface IDecoratorArgumentsDefinitionExtractorResult {
    definition: IExtractedApiDefinition;
    exprArguments?: ts.NodeArray<ts.Expression>;
}
export declare class ExtractionTransformer extends DecoratorTransformer<ts.MethodDeclaration, IExtractionTransformInfoBase> {
    private readonly onApiMethodExtracted;
    private readonly metadataManager;
    private readonly extractOnly;
    private paramTransformers;
    constructor(program: ts.Program, generator: tjs.JsonSchemaGenerator, transformInfo: IExtractionTransformInfoBase, onApiMethodExtracted: OnApiMethodExtractedHandler | undefined, metadataManager: IMetadataResolver, extractOnly?: boolean);
    visitNode(node: ts.MethodDeclaration): ts.MethodDeclaration;
    private parseApiMethodCallParameters;
    protected isDecoratedMethodDeclaration(node: ts.Node): node is ts.MethodDeclaration;
    protected getDecoratorArguments(decoratorArg: IExtractedApiDefinition, expression: ts.CallExpression): IDecoratorArgumentsDefinitionExtractorResult;
}
export {};
