"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HandlerMethodDecorator = void 0;
const DecoratorDefinition_1 = require("./DecoratorDefinition");
const HandlerTree_1 = require("./HandlerTree");
const TransformerMetadata_1 = require("./TransformerMetadata");
const Decorator_1 = require("./Decorator");
const CompilationError_1 = require("../Util/CompilationError");
class HandlerMethodDecorator extends Decorator_1.Decorator {
    constructor(definition) {
        super({
            ...definition,
            decoratorType: DecoratorDefinition_1.DecoratorType.Method,
        }, Decorator_1.DecoratorNodeType.Method);
    }
    get returnTypeRestriction() {
        return this.definition.returnTypeRestrictions;
    }
    getDecoratorTreeElement(parent, node, decorator, context) {
        let argumentResult = this.applyArguments(node, decorator, context);
        const apiMethod = (0, TransformerMetadata_1.getMetadataValueByDescriptor)(argumentResult.metadata, TransformerMetadata_1.BuiltinMetadata.ApiMethod);
        const route = (0, TransformerMetadata_1.getMetadataValueByDescriptor)(argumentResult.metadata, TransformerMetadata_1.BuiltinMetadata.Route);
        if (typeof apiMethod !== 'string') {
            throw new CompilationError_1.CompilationError('Handler did not have apiMethod metadata', node);
        }
        if (typeof route !== 'string') {
            throw new CompilationError_1.CompilationError('Handler did not have route metadata', node);
        }
        const decoratorTreeNode = {
            type: HandlerTree_1.HandlerTreeNodeType.Handler,
            decorator: this.definition,
            children: [],
            location: this.getNodeLocation(node),
            apiMethod,
            route,
            parent,
            metadata: argumentResult.metadata,
        };
        decoratorTreeNode.metadata = decoratorTreeNode.metadata.concat(context.metadataManager.getApiMetadataForApiMethod(decoratorTreeNode, node, this));
        return {
            decoratorTreeNode,
            transformedDecorator: argumentResult.decorator,
        };
    }
}
exports.HandlerMethodDecorator = HandlerMethodDecorator;
//# sourceMappingURL=HandlerMethodDecorator.js.map