import * as ts from 'typescript';
import { IDecoratorDefinition } from './DecoratorDefinition';
import { ApiMethod } from '../apiManagement';
import { IApiParamDefinition } from '../apiManagement/ApiDefinition';
import { ITransformerMetadataCollection } from './TransformerMetadata';
export declare enum HandlerTreeNodeType {
    Root = 0,
    Dependency = 1,
    DependencyProperty = 2,
    HandlerCollection = 3,
    Handler = 4,
    HandlerModifier = 5,
    HandlerParameter = 6
}
export interface IHandlerLocation {
    file: string;
    line: number;
    character: number;
    position: number;
}
export interface IHandlerTreeNodeBase extends ITransformerMetadataCollection {
    type: HandlerTreeNodeType;
    children: IHandlerTreeNode[];
    parent?: IHandlerTreeNode;
}
export interface IHandlerTreeNodeRoot extends IHandlerTreeNodeBase {
    type: HandlerTreeNodeType.Root;
    parent?: undefined;
}
export interface IHandlerTreeDecoratorNodeBase extends IHandlerTreeNodeBase {
    decorator: IDecoratorDefinition;
    location: IHandlerLocation;
}
export declare function isDecoratorNode(node: IHandlerTreeNode): node is IHandlerTreeDecoratorNode;
export interface IHandlerTreeNodeDependency extends IHandlerTreeDecoratorNodeBase {
    type: HandlerTreeNodeType.Dependency;
}
export declare const isDependencyNode: (node: IHandlerTreeNode) => node is IHandlerTreeNodeDependency;
export interface IHandlerTreeNodeDependencyProperty extends IHandlerTreeDecoratorNodeBase {
    type: HandlerTreeNodeType.DependencyProperty;
}
export declare const isDependencyPropertyNode: (node: IHandlerTreeNode) => node is IHandlerTreeNodeDependencyProperty;
export interface IHandlerTreeNodeHandlerCollection extends IHandlerTreeDecoratorNodeBase {
    type: HandlerTreeNodeType.HandlerCollection;
}
export declare const isHandlerCollectionNode: (node: IHandlerTreeNode) => node is IHandlerTreeNodeHandlerCollection;
export interface IHandlerTreeNodeHandler extends IHandlerTreeDecoratorNodeBase {
    type: HandlerTreeNodeType.Handler;
    apiMethod: ApiMethod;
    route: string;
}
export declare const isHandlerNode: (node: IHandlerTreeNode) => node is IHandlerTreeNodeHandler;
export interface IHandlerTreeNodeHandlerModifier extends IHandlerTreeDecoratorNodeBase {
    type: HandlerTreeNodeType.HandlerModifier;
}
export declare const isHandlerModifierNode: (node: IHandlerTreeNode) => node is IHandlerTreeNodeHandlerModifier;
export interface IHandlerTreeNodeParameter extends IHandlerTreeDecoratorNodeBase {
    type: HandlerTreeNodeType.HandlerParameter;
    paramDef: IApiParamDefinition;
}
export declare const isHandlerParameterNode: (node: IHandlerTreeNode) => node is IHandlerTreeNodeParameter;
export type IHandlerTreeDecoratorNode = IHandlerTreeNodeDependency | IHandlerTreeNodeDependencyProperty | IHandlerTreeNodeHandlerCollection | IHandlerTreeNodeHandler | IHandlerTreeNodeHandlerModifier | IHandlerTreeNodeParameter;
export type IHandlerTreeNode = IHandlerTreeNodeRoot | IHandlerTreeDecoratorNode;
export interface ITransformedTreeElement<T extends ts.Node> {
    transformedDecorator?: T;
    decoratorTreeNode: IHandlerTreeNode;
}
/**
 * Performs a depth-first search of the tree, yielding each node
 */
export declare function WalkTree(node: IHandlerTreeNode): Generator<IHandlerTreeNode, void, void>;
/**
 * Performs a depth-first search of a tree, only returning nodes that match checkFunc.
 * @param node
 * @param checkFunc
 */
export declare function WalkTreeByType<T extends any>(node: IHandlerTreeNode, checkFunc?: (node: any) => node is T): Generator<T, void, void>;
/**
 * Performs a depth-first search of a tree, only returning nodes that match checkFunc.
 * @param node
 * @param checkFunc
 */
export declare function WalkChildrenByType<T extends any>(node: IHandlerTreeNode, checkFunc?: (node: any) => node is T): Generator<T, void, void>;
export declare function MergeHandlerTreeRoots(a: IHandlerTreeNodeRoot, b: IHandlerTreeNodeRoot): IHandlerTreeNodeRoot;
