"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MergeHandlerTreeRoots = exports.WalkChildrenByType = exports.WalkTreeByType = exports.WalkTree = exports.isHandlerParameterNode = exports.isHandlerModifierNode = exports.isHandlerNode = exports.isHandlerCollectionNode = exports.isDependencyPropertyNode = exports.isDependencyNode = exports.isDecoratorNode = exports.HandlerTreeNodeType = void 0;
var HandlerTreeNodeType;
(function (HandlerTreeNodeType) {
    HandlerTreeNodeType[HandlerTreeNodeType["Root"] = 0] = "Root";
    HandlerTreeNodeType[HandlerTreeNodeType["Dependency"] = 1] = "Dependency";
    HandlerTreeNodeType[HandlerTreeNodeType["DependencyProperty"] = 2] = "DependencyProperty";
    HandlerTreeNodeType[HandlerTreeNodeType["HandlerCollection"] = 3] = "HandlerCollection";
    HandlerTreeNodeType[HandlerTreeNodeType["Handler"] = 4] = "Handler";
    HandlerTreeNodeType[HandlerTreeNodeType["HandlerModifier"] = 5] = "HandlerModifier";
    HandlerTreeNodeType[HandlerTreeNodeType["HandlerParameter"] = 6] = "HandlerParameter";
})(HandlerTreeNodeType || (exports.HandlerTreeNodeType = HandlerTreeNodeType = {}));
function isDecoratorNode(node) {
    return !!node.decorator;
}
exports.isDecoratorNode = isDecoratorNode;
exports.isDependencyNode = isNodeByType(HandlerTreeNodeType.Dependency);
exports.isDependencyPropertyNode = isNodeByType(HandlerTreeNodeType.DependencyProperty);
exports.isHandlerCollectionNode = isNodeByType(HandlerTreeNodeType.HandlerCollection);
exports.isHandlerNode = isNodeByType(HandlerTreeNodeType.Handler);
exports.isHandlerModifierNode = isNodeByType(HandlerTreeNodeType.HandlerModifier);
exports.isHandlerParameterNode = isNodeByType(HandlerTreeNodeType.HandlerParameter);
function isNodeByType(type) {
    return (node) => {
        return node.type === type;
    };
}
/**
 * Performs a depth-first search of the tree, yielding each node
 */
function* WalkTree(node) {
    yield* WalkTreeByType(node);
}
exports.WalkTree = WalkTree;
/**
 * Performs a depth-first search of a tree, only returning nodes that match checkFunc.
 * @param node
 * @param checkFunc
 */
function* WalkTreeByType(node, checkFunc) {
    for (const child of node.children) {
        if (!checkFunc || checkFunc(child)) {
            yield child;
        }
        yield* WalkTreeByType(child, checkFunc);
    }
}
exports.WalkTreeByType = WalkTreeByType;
/**
 * Performs a depth-first search of a tree, only returning nodes that match checkFunc.
 * @param node
 * @param checkFunc
 */
function* WalkChildrenByType(node, checkFunc) {
    for (const child of node.children) {
        if (!checkFunc || checkFunc(child)) {
            yield child;
        }
    }
}
exports.WalkChildrenByType = WalkChildrenByType;
function MergeHandlerTreeRoots(a, b) {
    if (!a) {
        return b;
    }
    if (!b) {
        return a;
    }
    return {
        type: HandlerTreeNodeType.Root,
        children: [
            ...a.children,
            ...b.children,
        ],
        metadata: [
            ...a.metadata,
            ...b.metadata,
        ],
    };
}
exports.MergeHandlerTreeRoots = MergeHandlerTreeRoots;
//# sourceMappingURL=HandlerTree.js.map