"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HandlerTreeDependencyUtil = void 0;
const DecoratorDefinition_1 = require("./DecoratorDefinition");
const HandlerTree_1 = require("./HandlerTree");
class HandlerTreeDependencyUtil {
    static CheckParentNodeDependencySatisfied(dependency, parent) {
        if (dependency.location !== DecoratorDefinition_1.DecoratorDependencyLocation.Parent) {
            throw new Error('Not a parent dependency');
        }
        do {
            if (!parent) {
                return false;
            }
            if (this.CheckDependency(dependency, parent)) {
                return true;
            }
        } while (parent = parent.parent);
        return false;
    }
    static CheckDependency(dependency, node) {
        if (!(0, HandlerTree_1.isDecoratorNode)(node)) {
            return false;
        }
        switch (dependency.type) {
            case DecoratorDefinition_1.DecoratorDependencyType.NameDependency:
                return node.decorator.magicFunctionName === dependency.dependency;
            case DecoratorDefinition_1.DecoratorDependencyType.ProviderDependency:
                return node.decorator.provider === dependency.dependency;
            default:
                throw new Error(`Unknown dependency type: ${dependency.type}`);
        }
    }
    static CheckPeerNodeDependencySatisfied(dependency, node) {
        if (dependency.location !== DecoratorDefinition_1.DecoratorDependencyLocation.Peer) {
            throw new Error('Not a peer dependency');
        }
        if (node.parent) {
            return !!node.parent.children.find(c => this.CheckDependency(dependency, c));
        }
        return false;
    }
}
exports.HandlerTreeDependencyUtil = HandlerTreeDependencyUtil;
//# sourceMappingURL=HandlerTreeDependencyUtil.js.map