import * as ts from 'typescript';
import { ITransformerMetadata } from "./TransformerMetadata";
import { IDecorationFunctionTransformInfoBase } from './DecoratorDefinition';
import { IHandlerTreeNodeHandler, IHandlerTreeNodeParameter, IHandlerTreeNodeHandlerCollection } from './HandlerTree';
export type ApiMethodMetadataGeneratorFunc = (method: IHandlerTreeNodeHandler, methodNode: ts.MethodDeclaration) => ITransformerMetadata[];
export type ApiMethodCollectionMetadataGeneratorFunc = (method: IHandlerTreeNodeHandlerCollection, collectionNode: ts.ClassDeclaration) => ITransformerMetadata[];
export type ApiMethodParamMetadataGeneratorFunc = (param: IHandlerTreeNodeParameter, paramNode: ts.ParameterDeclaration) => ITransformerMetadata[];
export interface IMetadataManager {
    addApiMethodCollectionMetadataGenerator(generator: ApiMethodCollectionMetadataGeneratorFunc): void;
    addApiMethodCollectionMetadataGenerator(generator: ApiMethodCollectionMetadataGeneratorFunc, decorator: IDecorationFunctionTransformInfoBase): void;
    removeApiMethodCollectionMetadataGenerator(generator: ApiMethodCollectionMetadataGeneratorFunc): void;
    removeApiMethodCollectionMetadataGenerator(generator: ApiMethodCollectionMetadataGeneratorFunc, decorator: IDecorationFunctionTransformInfoBase): void;
    addApiMethodMetadataGenerator(generator: ApiMethodMetadataGeneratorFunc): void;
    addApiMethodMetadataGenerator(generator: ApiMethodMetadataGeneratorFunc, decorator: IDecorationFunctionTransformInfoBase): void;
    removeApiMethodMetadataGenerator(generator: ApiMethodMetadataGeneratorFunc): void;
    removeApiMethodMetadataGenerator(generator: ApiMethodMetadataGeneratorFunc, decorator: IDecorationFunctionTransformInfoBase): void;
    addApiMethodParamMetadataGenerator(generator: ApiMethodParamMetadataGeneratorFunc): void;
    addApiMethodParamMetadataGenerator(generator: ApiMethodParamMetadataGeneratorFunc, decorator: IDecorationFunctionTransformInfoBase): void;
    removeApiMethodParamMetadataGenerator(generator: ApiMethodParamMetadataGeneratorFunc): void;
    removeApiMethodParamMetadataGenerator(generator: ApiMethodParamMetadataGeneratorFunc, decorator: IDecorationFunctionTransformInfoBase): void;
}
export interface IMetadataResolver {
    getApiMetadataForApiMethodCollection(collection: IHandlerTreeNodeHandlerCollection, collectionNode: ts.ClassDeclaration, decorator: IDecorationFunctionTransformInfoBase): ITransformerMetadata[];
    getApiMethodCollectionMetadataDecorators(): IDecorationFunctionTransformInfoBase[];
    getApiMetadataForApiMethod(method: IHandlerTreeNodeHandler, methodNode: ts.MethodDeclaration, decorator: IDecorationFunctionTransformInfoBase): ITransformerMetadata[];
    getApiMethodMetadataDecorators(): IDecorationFunctionTransformInfoBase[];
    getApiMetadataForApiMethodParam(param: IHandlerTreeNodeParameter, paramNode: ts.ParameterDeclaration, decorator: IDecorationFunctionTransformInfoBase): ITransformerMetadata[];
    getApiMethodParamMetadataDecorators(): IDecorationFunctionTransformInfoBase[];
}
export declare class MetadataManager implements IMetadataManager, IMetadataResolver {
    private apiMethodGenerators;
    private apiMethodCollectionGenerators;
    private apiMethodParamGenerators;
    addApiMethodCollectionMetadataGenerator(generator: ApiMethodCollectionMetadataGeneratorFunc): void;
    addApiMethodCollectionMetadataGenerator(generator: ApiMethodCollectionMetadataGeneratorFunc, decorator: IDecorationFunctionTransformInfoBase): void;
    getApiMethodCollectionMetadataDecorators(): IDecorationFunctionTransformInfoBase[];
    removeApiMethodCollectionMetadataGenerator(generator: ApiMethodCollectionMetadataGeneratorFunc): void;
    removeApiMethodCollectionMetadataGenerator(generator: ApiMethodCollectionMetadataGeneratorFunc, decorator: IDecorationFunctionTransformInfoBase): void;
    addApiMethodMetadataGenerator(generator: ApiMethodMetadataGeneratorFunc): void;
    addApiMethodMetadataGenerator(generator: ApiMethodMetadataGeneratorFunc, decorator: IDecorationFunctionTransformInfoBase): void;
    getApiMethodMetadataDecorators(): IDecorationFunctionTransformInfoBase[];
    removeApiMethodMetadataGenerator(generator: ApiMethodMetadataGeneratorFunc): void;
    removeApiMethodMetadataGenerator(generator: ApiMethodMetadataGeneratorFunc, decorator: IDecorationFunctionTransformInfoBase): void;
    addApiMethodParamMetadataGenerator(generator: ApiMethodParamMetadataGeneratorFunc): void;
    addApiMethodParamMetadataGenerator(generator: ApiMethodParamMetadataGeneratorFunc, decorator: IDecorationFunctionTransformInfoBase): void;
    removeApiMethodParamMetadataGenerator(generator: ApiMethodParamMetadataGeneratorFunc): void;
    removeApiMethodParamMetadataGenerator(generator: ApiMethodParamMetadataGeneratorFunc, decorator: IDecorationFunctionTransformInfoBase): void;
    getApiMethodParamMetadataDecorators(): IDecorationFunctionTransformInfoBase[];
    getApiMetadataForApiMethodCollection(collection: IHandlerTreeNodeHandlerCollection, collectionNode: ts.ClassDeclaration, decorator: IDecorationFunctionTransformInfoBase): ITransformerMetadata[];
    getApiMetadataForApiMethod(method: IHandlerTreeNodeHandler, methodNode: ts.MethodDeclaration, decorator: IDecorationFunctionTransformInfoBase | null): ITransformerMetadata[];
    getApiMetadataForApiMethodParam(param: IHandlerTreeNodeParameter, paramNode: ts.ParameterDeclaration, decorator: IDecorationFunctionTransformInfoBase | null): ITransformerMetadata[];
    private isMatchingDecoratorFuzzy;
    private isMatchingDecoratorExact;
}
