"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetadataManager = void 0;
const CollectionUtil_1 = require("../Util/CollectionUtil");
class MetadataManager {
    constructor() {
        this.apiMethodGenerators = new Map();
        this.apiMethodCollectionGenerators = new Map();
        this.apiMethodParamGenerators = new Map();
    }
    addApiMethodCollectionMetadataGenerator(generator, decorator) {
        CollectionUtil_1.CollectionUtil.addToMapSet(this.apiMethodCollectionGenerators, decorator || null, generator, this.isMatchingDecoratorExact);
    }
    getApiMethodCollectionMetadataDecorators() {
        return Array.from(this.apiMethodCollectionGenerators.keys())
            .filter(k => k !== null);
    }
    removeApiMethodCollectionMetadataGenerator(generator, decorator) {
        CollectionUtil_1.CollectionUtil.removeFromMapSet(this.apiMethodCollectionGenerators, decorator || null, generator, this.isMatchingDecoratorExact);
    }
    addApiMethodMetadataGenerator(generator, decorator) {
        CollectionUtil_1.CollectionUtil.addToMapSet(this.apiMethodGenerators, decorator || null, generator, this.isMatchingDecoratorExact);
    }
    getApiMethodMetadataDecorators() {
        return Array.from(this.apiMethodGenerators.keys())
            .filter(k => k !== null);
    }
    removeApiMethodMetadataGenerator(generator, decorator) {
        CollectionUtil_1.CollectionUtil.removeFromMapSet(this.apiMethodGenerators, decorator || null, generator, this.isMatchingDecoratorExact);
    }
    addApiMethodParamMetadataGenerator(generator, decorator) {
        CollectionUtil_1.CollectionUtil.addToMapSet(this.apiMethodParamGenerators, decorator || null, generator, this.isMatchingDecoratorExact);
    }
    removeApiMethodParamMetadataGenerator(generator, decorator) {
        CollectionUtil_1.CollectionUtil.removeFromMapSet(this.apiMethodParamGenerators, decorator || null, generator, this.isMatchingDecoratorExact);
    }
    getApiMethodParamMetadataDecorators() {
        return Array.from(this.apiMethodParamGenerators.keys())
            .filter(k => k !== null);
    }
    getApiMetadataForApiMethodCollection(collection, collectionNode, decorator) {
        const set = CollectionUtil_1.CollectionUtil.iterateSet(this.apiMethodCollectionGenerators, decorator, this.isMatchingDecoratorFuzzy);
        if (!set) {
            return [];
        }
        return Array.from(new Set(set))
            .reduce((prev, func) => {
            const result = func(collection, collectionNode);
            if (!Array.isArray(result)) {
                throw new Error(`ApiMethodCollectionMetadataGeneratorFunc return value must be an array`);
            }
            return prev.concat(result);
        }, []);
    }
    getApiMetadataForApiMethod(method, methodNode, decorator) {
        const set = CollectionUtil_1.CollectionUtil.iterateSet(this.apiMethodGenerators, decorator, this.isMatchingDecoratorFuzzy);
        if (!set) {
            return [];
        }
        return Array.from(new Set(set))
            .reduce((prev, func) => {
            const result = func(method, methodNode);
            if (!Array.isArray(result)) {
                throw new Error(`ApiMethodMetadataGeneratorFunc return value must be an array`);
            }
            return prev.concat(result);
        }, []);
    }
    getApiMetadataForApiMethodParam(param, paramNode, decorator) {
        const set = CollectionUtil_1.CollectionUtil.iterateSet(this.apiMethodParamGenerators, decorator, this.isMatchingDecoratorFuzzy);
        if (!set) {
            return [];
        }
        return Array.from(new Set(set))
            .reduce((prev, func) => {
            const result = func(param, paramNode);
            if (!Array.isArray(result)) {
                throw new Error(`ApiMethodParamMetadataGeneratorFunc return value must be an array`);
            }
            return prev.concat(result);
        }, []);
    }
    isMatchingDecoratorFuzzy(a, b) {
        if (a === b) {
            return true;
        }
        else if (a === null || b === null) {
            return true;
        }
        return a.magicFunctionName === b.magicFunctionName && a.indexTs === b.indexTs && a.provider === b.provider;
    }
    isMatchingDecoratorExact(a, b) {
        if (a === b) {
            return true;
        }
        else if (a === null || b === null) {
            return false;
        }
        return a.magicFunctionName === b.magicFunctionName && a.indexTs === b.indexTs && a.provider === b.provider;
    }
}
exports.MetadataManager = MetadataManager;
//# sourceMappingURL=MetadataManager.js.map