import * as ts from 'typescript';
import * as tjs from "typescript-json-schema";
import { InternalTypeDefinition } from '../apiManagement/InternalTypes';
import { DecoratorTransformer, IDecorationFunctionTransformInfoBase, ParamArgsInitializer, ExpressionWrapper } from './DecoratorTransformer';
import { ApiParamType } from '../apiManagement/ApiDefinition';
import { ClassConstructor } from '../decorators';
export declare type AllowableTypeStrings = 'object' | 'string' | 'number' | 'date' | 'boolean' | 'any';
export interface IParamDecoratorDefinition {
    allowableTypes: AllowableTypeStrings[];
    arguments: IParamDecoratorFunctionArg[];
}
export interface TransformerType {
    type: ApiParamType;
}
export declare type ParamDecoratorTransformerInfo = IParamDecoratorDefinition & IDecorationFunctionTransformInfoBase & TransformerType;
export interface IParamDecoratorFunctionArg {
    type: 'paramName' | 'validationFunc' | 'numberMin' | 'numberMax' | 'regexp';
    optional: boolean;
}
export declare class PropertyTransformer extends DecoratorTransformer<ts.PropertyDeclaration, ParamDecoratorTransformerInfo> {
    private readonly extractOnly;
    constructor(program: ts.Program, generator: tjs.JsonSchemaGenerator, transformInfo: ParamDecoratorTransformerInfo, extractOnly?: boolean);
    visitNode(node: ts.PropertyDeclaration): ts.PropertyDeclaration;
    getParamArgs(node: ts.PropertyDeclaration): {
        initializer: ExpressionWrapper;
        name: string;
        typedef: InternalTypeDefinition;
        optional?: boolean;
        description?: string;
        validationFunction?: import("../apiManagement/InternalTypes").ApiParamValidationFunction | ExpressionWrapper;
        regexp?: RegExp | ((t1: string) => RegExp) | ExpressionWrapper;
        typeref?: ClassConstructor<{}> | ExpressionWrapper;
    } | {
        name: string;
        typedef: InternalTypeDefinition;
        optional?: boolean;
        description?: string;
        initializer?: (() => any) | ExpressionWrapper;
        validationFunction?: import("../apiManagement/InternalTypes").ApiParamValidationFunction | ExpressionWrapper;
        regexp?: RegExp | ((t1: string) => RegExp) | ExpressionWrapper;
        typeref?: ClassConstructor<{}> | ExpressionWrapper;
    };
    getDecoratorArguments(decoratorArg: ParamArgsInitializer, expression: ts.CallExpression, transformInfo?: ParamDecoratorTransformerInfo): ParamArgsInitializer | undefined;
    private isNullOrUndefinedExpr;
    protected isDecoratedPropertyDeclaration(node: ts.Node, transformInfo?: IDecorationFunctionTransformInfoBase): node is ts.PropertyDeclaration;
}
