"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BuiltinMetadata = exports.getAllMetadataValues = exports.getAllMetadata = exports.getMetadataValueByDescriptor = exports.getMetadataByDescriptor = exports.getMetadataValue = exports.getMetadata = void 0;
function getMetadata(metadata, type, component, key) {
    return metadata.find(metadataSelector(type, component, key));
}
exports.getMetadata = getMetadata;
function getMetadataValue(metadata, type, component, key) {
    const found = getMetadata(metadata, type, component, key);
    if (found) {
        return found.value;
    }
    return undefined;
}
exports.getMetadataValue = getMetadataValue;
function getMetadataByDescriptor(metadata, descriptor) {
    return metadata.find(metadataSelector(descriptor.type, descriptor.component, descriptor.key));
}
exports.getMetadataByDescriptor = getMetadataByDescriptor;
function getMetadataValueByDescriptor(metadata, descriptor) {
    const found = getMetadata(metadata, descriptor.type, descriptor.component, descriptor.key);
    if (found) {
        return found.value;
    }
    return undefined;
}
exports.getMetadataValueByDescriptor = getMetadataValueByDescriptor;
function getAllMetadata(metadata, type, component, key) {
    return metadata.filter(metadataSelector(type, component, key));
}
exports.getAllMetadata = getAllMetadata;
function getAllMetadataValues(metadata, type, component, key) {
    return getAllMetadata(metadata, type, component, key).map(m => m.value);
}
exports.getAllMetadataValues = getAllMetadataValues;
function metadataSelector(type, component, key) {
    return m => {
        if (m.type !== type) {
            return false;
        }
        if (component && m.component !== component) {
            return false;
        }
        if (key && m.key !== key) {
            return false;
        }
        return true;
    };
}
class BuiltinMetadata {
    static ApiMethodWithValue(value) {
        return {
            ...BuiltinMetadata.ApiMethod,
            value,
        };
    }
}
exports.BuiltinMetadata = BuiltinMetadata;
BuiltinMetadata.BuiltinComponent = 'ts-api-decorators';
BuiltinMetadata.ApiMethod = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'apiMethod',
};
BuiltinMetadata.ApiMethodType = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'apiMethodType',
};
BuiltinMetadata.ApiMethodTypeHttp = {
    ...BuiltinMetadata.ApiMethodType,
    value: 'http',
};
BuiltinMetadata.Name = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'name',
};
BuiltinMetadata.FunctionArgumentName = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'functionargname',
};
BuiltinMetadata.SchemaRefOverride = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'schemarefoverride',
};
BuiltinMetadata.MimeType = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'mimeType',
};
BuiltinMetadata.Route = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'route',
};
BuiltinMetadata.ResponseCodes = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'responseCodes',
};
BuiltinMetadata.ApiProcessorStage = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'apiprocessorstage',
};
BuiltinMetadata.ApiProcessorScope = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'apiprocessorscope',
};
BuiltinMetadata.DependencyScope = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'dependencyScope',
};
BuiltinMetadata.NumberMin = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'numberMin',
};
BuiltinMetadata.NumberMax = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'numberMax',
};
BuiltinMetadata.UndefinedIfOmitted = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'undefinedIfOmitted',
};
BuiltinMetadata.ValidationRegExp = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'regexp',
};
BuiltinMetadata.ReturnSchema = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'returnSchema',
};
BuiltinMetadata.GetSchemaReturnSchema = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'returnSchema',
    value: {
        type: 'object',
        schema: {
            "$schema": "http://json-schema.org/draft-07/schema#",
        },
    }
};
BuiltinMetadata.DecoratorTypeArgType = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'decoratorTypeArgType',
};
BuiltinMetadata.ValidationFunction = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'validationFunc',
};
BuiltinMetadata.Typedef = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'typedef',
};
BuiltinMetadata.Typeref = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'typeref',
};
BuiltinMetadata.Optional = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'optional',
};
BuiltinMetadata.Initializer = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'initializer',
};
BuiltinMetadata.Description = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'description',
};
BuiltinMetadata.ParameterIndex = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'parameterIndex',
};
BuiltinMetadata.PropertyKey = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'propertyKey',
};
BuiltinMetadata.FormDataFieldName = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'formDataFieldName',
};
BuiltinMetadata.SchemaFormat = {
    type: "Builtin" /* IMetadataType.Builtin */,
    component: BuiltinMetadata.BuiltinComponent,
    key: 'schemaFormat',
};
//# sourceMappingURL=TransformerMetadata.js.map