"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransformerOpts = void 0;
const fs = require("fs");
const path = require("path");
const ts_node_1 = require("ts-node");
class TransformerOpts {
    /**
     * Parses transformer options in the root directory, if present.
     * @param rootDir
     */
    static ParseTransformerOpts(rootDir) {
        const optsFilePath = path.join(rootDir, TransformerOpts.TransformerOptsFile);
        if (fs.existsSync(optsFilePath)) {
            const strContents = fs.readFileSync(optsFilePath, { encoding: 'utf8' });
            let contents;
            try {
                contents = JSON.parse(strContents);
            }
            catch (e) {
                throw new Error(`${optsFilePath} contains invalid JSON`);
            }
            // TODO: file schema validation
            return new TransformerOpts(rootDir, {
                customDecorators: contents.customDecorators,
            });
        }
        return new TransformerOpts(rootDir, {});
    }
    constructor(rootDir, rootConfig) {
        this.rootDir = rootDir;
        this.rootConfig = rootConfig;
    }
    get customDecorators() {
        return this.rootConfig.customDecorators || [];
    }
    loadCustomDecorators() {
        if (!TransformerOpts.TsNodeRegistered) {
            (0, ts_node_1.register)();
            TransformerOpts.TsNodeRegistered = true;
        }
        return this.customDecorators
            .map(d => require(d))
            .reduce((p, c) => p.concat(Array.isArray(c) ? c : [c]), []);
    }
}
exports.TransformerOpts = TransformerOpts;
TransformerOpts.TransformerOptsFile = 'tsapiconfig.json';
TransformerOpts.TsNodeRegistered = false;
//# sourceMappingURL=TransformerOpts.js.map