import * as ts from 'typescript';
import { NodeWithTypeArguments } from 'typescript';
import { InternalTypeDefinition, BuiltinTypeNames } from '../apiManagement/InternalTypes';
export interface SymbolWithId extends ts.Symbol {
    id: number;
}
export interface SymbolWithParent extends ts.Symbol {
    parent: ts.Symbol;
}
export interface IntrinsicType extends ts.Type {
    intrinsicName: InternalTypeDefinition['type'];
}
export interface UnionType extends ts.Type {
    types: ts.Type[];
}
export interface IntersectionType extends ts.Type {
    types: ts.Type[];
}
export interface ArrayType extends ts.Type {
    elementType: ts.Type;
}
export interface WithJsDoc extends ts.Node {
    jsDoc: ts.JSDoc[];
}
export interface NamedNode extends ts.Node {
    name: ts.PropertyName;
}
export interface BuiltinSymbol extends ts.Symbol {
    name: BuiltinTypeNames;
}
export interface ParameterizedType extends ts.Type {
    resolvedTypeArguments: ts.Type[];
    typeArguments: ts.Type[];
}
export interface NodeWithElements extends ts.Node {
    elements: ts.Node[];
}
export declare function isNamedNode(n: ts.Node & Partial<NamedNode>): n is NamedNode;
export declare function isSymbolWithId(s: ts.Symbol & Partial<SymbolWithId>): s is SymbolWithId;
export declare function isSymbolWithParent(s: ts.Symbol & Partial<SymbolWithParent>): s is SymbolWithParent;
export declare function isBuiltinSymbol(s: ts.Symbol): s is BuiltinSymbol;
export declare function isIntrinsicType(s: ts.Type & Partial<IntrinsicType>): s is IntrinsicType;
export declare function nodeHasElements(s: ts.Node): s is NodeWithElements;
export declare function isUnionType(n: ts.TypeNode, s: ts.Type & Partial<UnionType>): s is UnionType;
export declare function isParameterizedType(t: ts.Type & Partial<ParameterizedType>): t is ParameterizedType;
export declare function isNodeWithTypeArguments(n: ts.Node & Partial<NodeWithTypeArguments>): n is NodeWithTypeArguments;
export declare function isIntersectionType(n: ts.TypeNode, s: ts.Type & Partial<IntersectionType>): s is IntersectionType;
export declare function isNodeWithJsDoc(n: ts.Node): n is WithJsDoc;
export declare function isNamedDeclaration(n: ts.Declaration): n is ts.NamedDeclaration;
export declare function isTypeWithSymbol<T extends object>(t: T): t is T & {
    symbol: ts.Symbol;
};
