import * as ts from 'typescript';
import * as tjs from 'typescript-json-schema';
import { TypeSerializer } from './TypeSerializer';
import { DecoratorNodeType, IDecorator } from './Decorator';
import { IHandlerTreeNodeRoot } from './HandlerTree';
import { ITransformContext } from './ITransformContext';
import { IMetadataResolver } from './MetadataManager';
import { IDecoratorResolver } from './IDecoratorResolver';
import { ITransformer } from './ITransformer';
export declare class TreeTransformer implements ITransformer {
    protected readonly program: ts.Program;
    protected readonly generator: tjs.JsonSchemaGenerator;
    protected readonly decorators: IDecoratorResolver;
    protected readonly applyTransformation: boolean;
    protected readonly typeChecker: ts.TypeChecker;
    protected readonly typeSerializer: TypeSerializer;
    protected readonly transformContext: ITransformContext;
    private readonly transformedNodes;
    private readonly rootNode;
    constructor(program: ts.Program, generator: tjs.JsonSchemaGenerator, decorators: IDecoratorResolver, applyTransformation: boolean, metadataManager: IMetadataResolver);
    get root(): IHandlerTreeNodeRoot;
    visitNode(node: ts.SourceFile, context: ts.TransformationContext): ts.SourceFile;
    visitNode(node: ts.Node, context: ts.TransformationContext): ts.Node;
    private visitNodeChildren;
    private visitNodeInTreeContext;
    private getDecoratorTreeElement;
    private cacheDecoratorTreeElement;
    private visitNodeChildrenInTreeContext;
    private nodeHasDecorators;
    /**
     * Returns true if this node type could potentially contain a class
     * @param node
     */
    private isPotentialClassContainer;
    getTypeForNode(node: ts.Node): DecoratorNodeType;
    protected isArgumentDecoratorCallExpression(decoratorExpression: ts.Decorator['expression'], definition: IDecorator): decoratorExpression is ts.CallExpression;
    protected isMatchingDeclaration(declaration: ts.NamedDeclaration, definition: IDecorator): boolean;
    protected isArgumentDecoratorExpression(decoratorExpression: ts.Decorator['expression'], definition: IDecorator): boolean;
}
