"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const tjs = require("typescript-json-schema");
const QueryParams_1 = require("../../decorators/QueryParams");
const BodyParams_1 = require("../../decorators/BodyParams");
const ExtractionTransformer_1 = require("../ExtractionTransformer");
const __1 = require("../..");
const transformer_1 = require("./transformer");
const MetadataManager_1 = require("../MetadataManager");
const OpenApi_1 = require("../OpenApi");
const decorators_1 = require("../../decorators");
const TJSGeneratorUtil_1 = require("../../Util/TJSGeneratorUtil");
function transformer(program, opts) {
    const generator = tjs.buildGenerator(program, TJSGeneratorUtil_1.TJSDefaultOptions());
    let metadataManager = opts.metadataManager;
    if (!metadataManager) {
        const mgr = new MetadataManager_1.MetadataManager();
        if (!opts.disableOpenApiMetadata) {
            OpenApi_1.OpenApiMetadataExtractors.RegisterMetadataExtractors(mgr);
        }
        metadataManager = mgr;
    }
    const indexTs = path.join('decorators/API');
    const queryParamIndexTs = path.join('decorators/QueryParams');
    const bodyParamIndexTs = path.join('decorators/BodyParams');
    const headerParamIndexTs = path.join('decorators/HeaderParams');
    const parameterTypes = [
        transformer_1.getQueryParamDecoratorInfo(QueryParams_1.ApiQueryParam.name, queryParamIndexTs),
        transformer_1.getQueryParamDecoratorInfo(QueryParams_1.ApiQueryParamString.name, queryParamIndexTs),
        transformer_1.getQueryParamDecoratorInfo(QueryParams_1.ApiQueryParamNumber.name, queryParamIndexTs),
        transformer_1.getBodyParamDecoratorInfo(BodyParams_1.ApiBodyParam.name, bodyParamIndexTs),
        transformer_1.getBodyParamDecoratorInfo(BodyParams_1.ApiBodyParamString.name, bodyParamIndexTs),
        transformer_1.getBodyParamDecoratorInfo(BodyParams_1.ApiBodyParamNumber.name, bodyParamIndexTs),
        transformer_1.getHeaderParamDecoratorInfo(decorators_1.ApiHeaderParam.name, headerParamIndexTs),
        transformer_1.getHeaderParamDecoratorInfo(decorators_1.ApiHeaderParamNumber.name, headerParamIndexTs),
        transformer_1.getHeaderParamDecoratorInfo(decorators_1.ApiHeaderParamString.name, headerParamIndexTs),
    ];
    const transformers = [
        // GET
        new ExtractionTransformer_1.ExtractionTransformer(program, generator, Object.assign({}, decorators_1.GetApiMethodDecorator(__1.ApiGetMethod.name), { parameterTypes,
            indexTs }), opts.onApiMethodExtracted, metadataManager),
        // POST
        new ExtractionTransformer_1.ExtractionTransformer(program, generator, Object.assign({}, decorators_1.GetApiMethodDecorator(decorators_1.ApiPostMethod.name), { parameterTypes,
            indexTs }), opts.onApiMethodExtracted, metadataManager),
        // PUT
        new ExtractionTransformer_1.ExtractionTransformer(program, generator, Object.assign({}, decorators_1.GetApiMethodDecorator(decorators_1.ApiPutMethod.name), { parameterTypes,
            indexTs }), opts.onApiMethodExtracted, metadataManager),
        // DELETE
        new ExtractionTransformer_1.ExtractionTransformer(program, generator, Object.assign({}, decorators_1.GetApiMethodDecorator(decorators_1.ApiDeleteMethod.name), { parameterTypes,
            indexTs }), opts.onApiMethodExtracted, metadataManager),
    ];
    return (context) => (file) => {
        for (const transformer of transformers) {
            file = transformer.visitNodeAndChildren(file, context);
        }
        return file;
    };
}
exports.default = transformer;
//# sourceMappingURL=extractionTransformer.js.map