import * as ts from 'typescript';
import { IDecoratorResolver } from '../IDecoratorResolver';
import { DecoratorResolver } from '../DecoratorResolver';
import { IHandlerTreeNodeRoot } from '../HandlerTree';
import { TransformerOpts } from '../TransformerOpts';
export type OnTreeExtractedHandler = (error: any, treeRoot: IHandlerTreeNodeRoot) => void;
export interface ITransformerArguments {
    /**
     * A resolver to use when performing transformations.
     */
    decoratorResolver?: IDecoratorResolver;
    /**
     * If `decoratorResolver` is specified, whether to register the builtin decorator suite.
     */
    registerBuiltindecorators?: boolean;
    /**
     * Whether to apply transformers or just extract data. Defaults to true.
     */
    applyTransformation?: boolean;
    /**
     * A method to be called when the Handler Tree is extracted.
     */
    onTreeExtracted?: OnTreeExtractedHandler;
    /**
     * Program-specified options for the transformer.
     */
    transformerOpts?: TransformerOpts;
}
export default function transformer(program: ts.Program, args?: ITransformerArguments): ts.TransformerFactory<ts.SourceFile>;
export declare function getDefaultDecoratorResolver(): DecoratorResolver;
export declare function loadTransformerOpts(rootDir: string): TransformerOpts;
export declare function registerDefaultDecorators(resolver: IDecoratorResolver): void;
