"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerDefaultDecorators = exports.loadTransformerOpts = exports.getDefaultDecoratorResolver = void 0;
const tjs = require("typescript-json-schema");
const QueryParams_1 = require("../../decorators/QueryParams");
const BodyParams_1 = require("../../decorators/BodyParams");
const TJSGeneratorUtil_1 = require("../../Util/TJSGeneratorUtil");
const __1 = require("../..");
const MetadataManager_1 = require("../MetadataManager");
const OpenApi_1 = require("../OpenApi");
const HeaderParams_1 = require("../../decorators/HeaderParams");
const PathParams_1 = require("../../decorators/PathParams");
const DependencyParams_1 = require("../../decorators/DependencyParams");
const DecoratorResolver_1 = require("../DecoratorResolver");
const decorators_1 = require("../../decorators");
const TreeTransformer_1 = require("../TreeTransformer");
const TransformerOpts_1 = require("../TransformerOpts");
const ApiProcessing_1 = require("../../decorators/ApiProcessing");
function transformer(program, args = {}) {
    const generator = tjs.buildGenerator(program, {
        ...(0, TJSGeneratorUtil_1.TJSDefaultOptions)(),
        ignoreErrors: true,
    });
    let metadataManager = new MetadataManager_1.MetadataManager();
    OpenApi_1.OpenApiMetadataExtractors.RegisterMetadataExtractors(metadataManager);
    if (!args.decoratorResolver) {
        args.decoratorResolver = getDefaultDecoratorResolver();
    }
    else if (args.registerBuiltindecorators) {
        registerDefaultDecorators(args.decoratorResolver);
    }
    if (!args.transformerOpts) {
        const { rootDir } = program.getCompilerOptions();
        if (rootDir) {
            args.transformerOpts = loadTransformerOpts(rootDir);
            if (args.transformerOpts.loadCustomDecorators) {
                args.decoratorResolver.addDecorators(args.transformerOpts.loadCustomDecorators());
            }
        }
    }
    if (typeof args.applyTransformation !== 'boolean') {
        args.applyTransformation = true;
    }
    return (context) => (file) => {
        const transformer = new TreeTransformer_1.TreeTransformer(program, generator, args.decoratorResolver, args.applyTransformation, metadataManager);
        file = transformer.visitNode(file, context);
        if (args.onTreeExtracted) {
            args.onTreeExtracted(null, transformer.root);
        }
        return file;
    };
}
exports.default = transformer;
function getDefaultDecoratorResolver() {
    const resolver = new DecoratorResolver_1.DecoratorResolver();
    registerDefaultDecorators(resolver);
    return resolver;
}
exports.getDefaultDecoratorResolver = getDefaultDecoratorResolver;
function loadTransformerOpts(rootDir) {
    return TransformerOpts_1.TransformerOpts.ParseTransformerOpts(rootDir);
}
exports.loadTransformerOpts = loadTransformerOpts;
function registerDefaultDecorators(resolver) {
    const decorators = [
        (0, decorators_1.GetApiDecorator)(decorators_1.Api),
        (0, __1.GetApiMethodDecorator)(__1.ApiGetMethod),
        (0, __1.GetApiMethodDecorator)(__1.ApiPutMethod),
        (0, __1.GetApiMethodDecorator)(__1.ApiPostMethod),
        (0, __1.GetApiMethodDecorator)(__1.ApiDeleteMethod),
        (0, __1.GetApiMethodDecorator)(decorators_1.ApiGetSchemaMethod),
        (0, QueryParams_1.GetQueryParamDecorator)(QueryParams_1.ApiQueryParam),
        (0, QueryParams_1.GetQueryParamDecorator)(QueryParams_1.ApiQueryParamString),
        (0, QueryParams_1.GetQueryParamDecorator)(QueryParams_1.ApiQueryParamNumber),
        (0, BodyParams_1.GetBodyParamDecorator)(BodyParams_1.ApiBodyParam),
        (0, BodyParams_1.GetBodyParamDecorator)(BodyParams_1.ApiBodyParamString),
        (0, BodyParams_1.GetBodyParamDecorator)(BodyParams_1.ApiBodyParamNumber),
        (0, BodyParams_1.GetBodyParamDecorator)(BodyParams_1.ApiBodyParamStream),
        (0, BodyParams_1.GetBodyParamDecorator)(BodyParams_1.ApiBodyParamRawString),
        // GetBodyParamDecorator(ApiBodyParamMultipartFormFileName),
        // GetBodyParamDecorator(ApiBodyParamMultipartFormFileString),
        // GetBodyParamDecorator(ApiBodyParamMultipartFormFileStream),
        (0, HeaderParams_1.GetHeaderParamDecorator)(HeaderParams_1.ApiHeaderParam),
        (0, HeaderParams_1.GetHeaderParamDecorator)(HeaderParams_1.ApiHeaderParamNumber),
        (0, HeaderParams_1.GetHeaderParamDecorator)(HeaderParams_1.ApiHeaderParamString),
        (0, PathParams_1.GetPathParamDecorator)(PathParams_1.ApiPathParam),
        (0, PathParams_1.GetPathParamDecorator)(PathParams_1.ApiPathParamNumber),
        (0, PathParams_1.GetPathParamDecorator)(PathParams_1.ApiPathParamString),
        (0, DependencyParams_1.GetDependencyParamDecorator)(DependencyParams_1.ApiDependency),
        (0, DependencyParams_1.GetDependencyParamDecorator)(DependencyParams_1.ApiInjectedDependency),
        (0, DependencyParams_1.GetDependencyParamDecorator)(DependencyParams_1.ApiInjectedDependencyParam),
        (0, ApiProcessing_1.GetApiProcessorDecorator)(ApiProcessing_1.ApiProcessor),
    ];
    for (const decorator of decorators) {
        resolver.addDecorator(decorator);
    }
}
exports.registerDefaultDecorators = registerDefaultDecorators;
//# sourceMappingURL=transformer.js.map